@@ -1,7 +1,7 @@
 
 #include <config.h>
 
-#ident "$Id: commonio.c,v 1.32 2006/05/07 18:32:51 kloczek Exp $"
+#ident "$Id: commonio.c 1892 2008-03-08 22:52:44Z nekral-guest $"
 
 #include "defines.h"
 #include <sys/stat.h>
@@ -12,11 +12,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <signal.h>
-#include <pwd.h>
-#include <nscd.h>
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
-#endif
+#include "nscd.h"
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
 static security_context_t old_context = NULL;
@@ -35,6 +31,9 @@ static int name_is_nis (const char *);
 static int write_all (const struct commonio_db *);
 static struct commonio_entry *find_entry_by_name (struct commonio_db *,
 						  const char *);
+static struct commonio_entry *next_entry_by_name (struct commonio_db *,
+                                                  struct commonio_entry *pos,
+                                                  const char *);
 
 static int lock_count = 0;
 static int nscd_need_reload = 0;
@@ -50,7 +49,7 @@ int lrename (const char *old, const char *new)
 	int res;
 
 #if defined(S_ISLNK)
-	struct stat sb = { 0 };
+	struct stat sb;
 	if (lstat (new, &sb) == 0 && S_ISLNK (sb.st_mode)) {
 		if (realpath (new, resolved_path) == NULL) {
 			perror ("realpath in lrename()");
@@ -392,6 +391,9 @@ static int name_is_nis (const char *n)
 #if KEEP_NIS_AT_END
 static void add_one_entry_nis (struct commonio_db *, struct commonio_entry *);
 
+/*
+ * Insert an entry between the regular entries, and the NIS entries.
+ */
 static void
 add_one_entry_nis (struct commonio_db *db, struct commonio_entry *newp)
 {
@@ -457,6 +459,10 @@ int commonio_open (struct commonio_db *db, int mode)
 		}
 		return 0;
 	}
+
+	/* Do not inherit fd in spawned processes (e.g. nscd) */
+	fcntl(fileno(db->fp), F_SETFD, FD_CLOEXEC);
+
 #ifdef WITH_SELINUX
 	db->scontext = NULL;
 	if ((is_selinux_enabled () > 0) && (!db->readonly)) {
@@ -513,6 +519,9 @@ int commonio_open (struct commonio_db *db, int mode)
 	if (ferror (db->fp))
 		goto cleanup_errno;
 
+	if (db->ops->open_hook && !db->ops->open_hook ())
+		goto cleanup_errno;
+
 	db->isopen = 1;
 	return 1;
 
@@ -623,6 +632,11 @@ int commonio_sort_wrt (struct commonio_db *shadow, struct commonio_db *passwd)
 	return 0;
 }
 
+/*
+ * write_all - Write the database to its file.
+ *
+ * It returns 0 if all the entries could be writen correctly.
+ */
 static int write_all (const struct commonio_db *db)
 {
 	const struct commonio_entry *p;
@@ -662,6 +676,9 @@ int commonio_close (struct commonio_db *db)
 		goto success;
 	}
 
+	if (db->ops->close_hook && !db->ops->close_hook ())
+		goto fail;
+
 	memzero (&sb, sizeof sb);
 	if (db->fp) {
 		if (fstat (fileno (db->fp), &sb)) {
@@ -671,9 +688,7 @@ int commonio_close (struct commonio_db *db)
 		}
 #ifdef WITH_SELINUX
 		if (db->scontext != NULL) {
-			int stat = getfscreatecon (&old_context);
-
-			if (stat < 0) {
+			if (getfscreatecon (&old_context) < 0) {
 				errors++;
 				goto fail;
 			}
@@ -761,14 +776,17 @@ int commonio_close (struct commonio_db *db)
 	return errors == 0;
 }
 
-
-static struct commonio_entry *find_entry_by_name (struct commonio_db *db,
-						  const char *name)
+static struct commonio_entry *next_entry_by_name (struct commonio_db *db,
+                                                  struct commonio_entry *pos,
+                                                  const char *name)
 {
 	struct commonio_entry *p;
 	void *ep;
 
-	for (p = db->head; p; p = p->next) {
+	if (NULL == pos)
+		return NULL;
+
+	for (p = pos; p; p = p->next) {
 		ep = p->eptr;
 		if (ep && strcmp (db->ops->getname (ep), name) == 0)
 			break;
@@ -776,6 +794,12 @@ static struct commonio_entry *find_entry_by_name (struct commonio_db *db,
 	return p;
 }
 
+static struct commonio_entry *find_entry_by_name (struct commonio_db *db,
+						  const char *name)
+{
+	return next_entry_by_name(db, db->head, name);
+}
+
 
 int commonio_update (struct commonio_db *db, const void *eptr)
 {
@@ -792,6 +816,11 @@ int commonio_update (struct commonio_db *db, const void *eptr)
 	}
 	p = find_entry_by_name (db, db->ops->getname (eptr));
 	if (p) {
+		if (next_entry_by_name (db, p->next, db->ops->getname (eptr)))
+		{
+			fprintf (stderr, _("Multiple entries named '%s' in %s. Please fix this with pwck or grpck.\n"), db->ops->getname (eptr), db->filename);
+			return 0;
+		}
 		db->ops->free (p->eptr);
 		p->eptr = nentry;
 		p->changed = 1;
@@ -841,7 +870,9 @@ void commonio_del_entry (struct commonio_db *db, const struct commonio_entry *p)
 	db->changed = 1;
 }
 
-
+/*
+ * commonio_remove - Remove the entry of the given name from the database.
+ */
 int commonio_remove (struct commonio_db *db, const char *name)
 {
 	struct commonio_entry *p;
@@ -855,6 +886,10 @@ int commonio_remove (struct commonio_db *db, const char *name)
 		errno = ENOENT;
 		return 0;
 	}
+	if (next_entry_by_name (db, p->next, name)) {
+		fprintf (stderr, _("Multiple entries named '%s' in %s. Please fix this with pwck or grpck.\n"), name, db->filename);
+		return 0;
+	}
 
 	commonio_del_entry (db, p);
 
@@ -867,7 +902,15 @@ int commonio_remove (struct commonio_db *db, const char *name)
 	return 1;
 }
 
-
+/*
+ * commonio_locate - Find the first entry with the specified name in
+ *                   the database.
+ *
+ *	If found, it returns the entry and set the cursor of the database to
+ *	that entry.
+ *
+ *	Otherwise, it returns NULL.
+ */
 const void *commonio_locate (struct commonio_db *db, const char *name)
 {
 	struct commonio_entry *p;
@@ -885,7 +928,11 @@ const void *commonio_locate (struct commonio_db *db, const char *name)
 	return p->eptr;
 }
 
-
+/*
+ * commonio_rewind - Restore the database cursor to the first entry.
+ *
+ * It returns 0 on error, 1 on success.
+ */
 int commonio_rewind (struct commonio_db *db)
 {
 	if (!db->isopen) {
@@ -896,7 +943,11 @@ int commonio_rewind (struct commonio_db *db)
 	return 1;
 }
 
-
+/*
+ * commonio_next - Return the next entry of the specified database
+ *
+ * It returns the next entry, or NULL if no other entries could be found.
+ */
 const void *commonio_next (struct commonio_db *db)
 {
 	void *eptr;
