@@ -1,4 +1,4 @@
-/* $Id: defines.h 1342 2007-11-10 23:46:11Z nekral-guest $ */
+/* $Id: defines.h 1761 2008-02-03 17:53:21Z nekral-guest $ */
 /* some useful defines */
 
 #ifndef _DEFINES_H_
@@ -101,19 +101,11 @@ char *strchr (), *strrchr (), *strtok ();
  * Possible cases:
  * - /usr/include/shadow.h exists and includes the shadow group stuff.
  * - /usr/include/shadow.h exists, but we use our own gshadow.h.
- * - /usr/include/shadow.h doesn't exist, use our own shadow.h and gshadow.h.
  */
-#if HAVE_SHADOW_H
 #include <shadow.h>
 #if defined(SHADOWGRP) && !defined(GSHADOW)
 #include "gshadow_.h"
 #endif
-#else				/* not HAVE_SHADOW_H */
-#include "shadow_.h"
-#ifdef SHADOWGRP
-#include "gshadow_.h"
-#endif
-#endif				/* not HAVE_SHADOW_H */
 
 #include <limits.h>
 
@@ -150,13 +142,15 @@ char *strchr (), *strrchr (), *strtok ();
    syslogd should log the current system time for each event, and not
    trust the formatted time received from the unix domain (or worse,
    UDP) socket.  -MM */
+/* Avoid translated PAM error messages: Set LC_ALL to "C".
+ * --Nekral */
 #define SYSLOG(x)							\
 	do {								\
 		char *saved_locale = setlocale(LC_ALL, NULL);		\
 		if (saved_locale)					\
 			saved_locale = strdup(saved_locale);		\
 		if (saved_locale)					\
-			setlocale(LC_TIME, "C");			\
+			setlocale(LC_ALL, "C");			\
 		syslog x ;						\
 		if (saved_locale) {					\
 			setlocale(LC_ALL, saved_locale);		\
@@ -270,19 +264,6 @@ char *strchr (), *strrchr (), *strtok ();
 #define STRFCPY(A,B) \
 	(strncpy((A), (B), sizeof(A) - 1), (A)[sizeof(A) - 1] = '\0')
 
-/* get rid of a few ugly repeated #ifdefs in pwent.c and grent.c */
-/* XXX - this is ugly too, configure should test it and not check for
-   any hardcoded system names, if possible.  --marekm */
-#if defined(AIX) || defined(__linux__)
-#define SETXXENT_TYPE void
-#define SETXXENT_RET(x) return
-#define SETXXENT_TEST(x) x; if (0)	/* compiler should optimize this away */
-#else
-#define SETXXENT_TYPE int
-#define SETXXENT_RET(x) return(x)
-#define SETXXENT_TEST(x) if (x)
-#endif
-
 #ifndef PASSWD_FILE
 #define PASSWD_FILE "/etc/passwd"
 #endif
@@ -301,11 +282,6 @@ char *strchr (), *strrchr (), *strtok ();
 #endif
 #endif
 
-#define PASSWD_PAG_FILE  PASSWD_FILE ".pag"
-#define GROUP_PAG_FILE   GROUP_FILE  ".pag"
-#define SHADOW_PAG_FILE  SHADOW_FILE ".pag"
-#define SGROUP_PAG_FILE  SGROUP_FILE ".pag"
-
 #ifndef NULL
 #define NULL ((void *) 0)
 #endif
@@ -318,10 +294,6 @@ extern char *strerror ();
 # endif
 #endif
 
-#ifndef HAVE_SNPRINTF
-#include "snprintf.h"
-#endif
-
 /*
  * string to use for the pw_passwd field in /etc/passwd when using
  * shadow passwords - most systems use "x" but there are a few
@@ -342,4 +314,11 @@ extern char *strerror ();
 #include <libaudit.h>
 #endif
 
+/* To be used for verified unused parameters */
+#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
+# define unused __attribute__((unused))
+#else
+# define unused
+#endif
+
 #endif				/* _DEFINES_H_ */
