@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: getdef.c 1442 2007-11-23 00:07:59Z nekral-guest $"
+#ident "$Id: getdef.c 1862 2008-02-25 21:06:30Z nekral-guest $"
 
 #include "prototypes.h"
 #include "defines.h"
@@ -77,6 +77,10 @@ static struct itemdef def_table[] = {
 	{"SHA_CRYPT_MIN_ROUNDS", NULL},
 	{"SULOG_FILE", NULL},
 	{"SU_NAME", NULL},
+	{"SYS_GID_MAX", NULL},
+	{"SYS_GID_MIN", NULL},
+	{"SYS_UID_MAX", NULL},
+	{"SYS_UID_MIN", NULL},
 	{"TTYGROUP", NULL},
 	{"TTYPERM", NULL},
 	{"TTYTYPE_FILE", NULL},
@@ -257,8 +261,8 @@ int putdef_str (const char *name, const char *value)
 	 * Save off the value.
 	 */
 	if ((cp = strdup (value)) == NULL) {
-		fprintf (stderr,
-			 _("Could not allocate space for config info.\n"));
+		fputs (_("Could not allocate space for config info.\n"),
+		       stderr);
 		SYSLOG ((LOG_ERR, "could not allocate space for config info"));
 		return -1;
 	}
