@@ -1,7 +1,7 @@
-extern struct group *__gr_dup (const struct group *);
-extern void __gr_set_changed (void);
+#include <grp.h>
 extern int gr_close (void);
 extern const struct group *gr_locate (const char *);
+extern const struct group *gr_locate_gid (gid_t gid);
 extern int gr_lock (void);
 extern int gr_name (const char *);
 extern const struct group *gr_next (void);
