@@ -30,7 +30,7 @@
 #include <config.h>
 
 #ifndef USE_PAM
-#ident "$Id: pwauth.c,v 1.23 2006/06/24 13:17:18 kloczek Exp $"
+#ident "$Id: pwauth.c 1596 2008-01-01 14:32:41Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -207,4 +207,6 @@ pw_auth (const char *cipher, const char *user, int reason, const char *input)
 		strzero (clear);
 	return retval;
 }
+#else				/* !USE_PAM */
+extern int errno;		/* warning: ANSI C forbids an empty source file */
 #endif				/* !USE_PAM */
