@@ -1,7 +1,7 @@
-extern struct passwd *__pw_dup (const struct passwd *);
-extern void __pw_set_changed (void);
+#include <pwd.h>
 extern int pw_close (void);
 extern const struct passwd *pw_locate (const char *);
+extern const struct passwd *pw_locate_uid (uid_t uid);
 extern int pw_lock (void);
 extern int pw_name (const char *);
 extern const struct passwd *pw_next (void);
