@@ -1,13 +1,11 @@
 
 #include <config.h>
 
-#ident "$Id: shadowio.c 1442 2007-11-23 00:07:59Z nekral-guest $"
+#ident "$Id: shadowio.c 1657 2008-01-06 11:59:01Z nekral-guest $"
 
 #include "prototypes.h"
 #include "defines.h"
-#ifdef HAVE_SHADOW_H
-# include <shadow.h>
-#endif
+#include <shadow.h>
 #include <stdio.h>
 #include "commonio.h"
 #include "shadowio.h"
@@ -143,7 +141,5 @@ void __spw_del_entry (const struct commonio_entry *ent)
 /* Sort with respect to passwd ordering. */
 int spw_sort ()
 {
-	extern struct commonio_db *__pw_get_db ();
-
 	return commonio_sort_wrt (&shadow_db, __pw_get_db ());
 }
