@@ -1,13 +1,11 @@
 
 #include <config.h>
 
-#ident "$Id: shadowmem.c 1443 2007-11-23 19:44:57Z nekral-guest $"
+#ident "$Id: shadowmem.c 1650 2008-01-05 16:33:43Z nekral-guest $"
 
 #include "prototypes.h"
 #include "defines.h"
-#ifdef HAVE_SHADOW_H
-# include <shadow.h>
-#endif
+#include <shadow.h>
 #include <stdio.h>
 #include "shadowio.h"
 
