@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: chowntty.c,v 1.13 2005/08/31 17:24:57 kloczek Exp $"
+#ident "$Id: chowntty.c 1397 2007-11-18 23:15:26Z nekral-guest $"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -77,7 +77,7 @@ void chown_tty (const char *tty, const struct passwd *info)
 		gid = info->pw_gid;
 	else if (group[0] >= '0' && group[0] <= '9')
 		gid = atoi (group);
-	else if ((grent = getgrnam (group)))
+	else if ((grent = getgrnam (group))) /* local, no need for xgetgrnam */
 		gid = grent->gr_gid;
 	else
 		gid = info->pw_gid;
