@@ -221,8 +221,8 @@
 #define yytable  gd_yytable
 #define yycheck  gd_yycheck
 
-static int yylex ();
-static int yyerror (char *s);
+static int yylex (void);
+static int yyerror (const char *s);
 
 #define EPOCH		1970
 #define HOUR(x)		((x) * 60)
@@ -2349,8 +2349,7 @@ static TABLE const MilitaryTable[] = {
 
 
 
-/* ARGSUSED */
-static int yyerror (char *s)
+static int yyerror (unused const char *s)
 {
   return 0;
 }
@@ -2516,7 +2515,7 @@ static int LookupWord (char *buff)
 }
 
 static int
-yylex ()
+yylex (void)
 {
   register char c;
   register char *p;
