@@ -10,7 +10,7 @@
 
 #include <config.h>
 
-#ident "$Id: myname.c,v 1.4 2005/08/31 17:24:57 kloczek Exp $"
+#ident "$Id: myname.c 1397 2007-11-18 23:15:26Z nekral-guest $"
 
 #include "defines.h"
 #include <pwd.h>
@@ -31,8 +31,9 @@ struct passwd *get_my_pwent (void)
 	 * XXX - when running from su, will return the current user (not
 	 * the original user, like getlogin() does).  Does this matter?
 	 */
-	if (cp && *cp && (pw = getpwnam (cp)) && pw->pw_uid == ruid)
+	if (cp && *cp && (pw = xgetpwnam (cp)) && pw->pw_uid == ruid)
 		return pw;
 
-	return getpwuid (ruid);
+	return xgetpwuid (ruid);
 }
+
