@@ -1,17 +1,15 @@
 #include <config.h>
 
-#ident "$Id: pwdcheck.c,v 1.9 2005/10/19 15:21:07 kloczek Exp $"
+#ident "$Id: pwdcheck.c 1650 2008-01-05 16:33:43Z nekral-guest $"
 
-#include <pwd.h>
 #include <stdio.h>
 #include "prototypes.h"
 #include "defines.h"
-#include "pwauth.h"
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
-#endif
 #ifdef USE_PAM
 #include "pam_defs.h"
+#else
+#include <shadow.h>
+#include "pwauth.h"
 #endif
 #define WRONGPWD2	"incorrect password for `%s'"
 void passwd_check (const char *user, const char *passwd, const char *progname)
@@ -46,7 +44,7 @@ void passwd_check (const char *user, const char *passwd, const char *progname)
 
 	struct spwd *sp;
 
-	if ((sp = getspnam (user)))
+	if ((sp = getspnam (user))) /* !USE_PAM, no need for xgetspnam */
 		passwd = sp->sp_pwdp;
 	endspent ();
 	if (pw_auth (passwd, user, PW_LOGIN, (char *) 0) != 0) {
