@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: chsh.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: chsh.c 1751 2008-02-03 16:28:03Z nekral-guest $"
 
 #include <fcntl.h>
 #include <getopt.h>
@@ -41,9 +41,6 @@
 #include <selinux/selinux.h>
 #include <selinux/av_permissions.h>
 #endif
-#ifdef HAVE_SHADOW_H
-#include <shadow.h>
-#endif
 #include "defines.h"
 #include "exitcodes.h"
 #include "getdef.h"
@@ -63,6 +60,11 @@
 static char *Prog;		/* Program name */
 static int amroot;		/* Real UID is root */
 static char loginsh[BUFSIZ];	/* Name of new login shell */
+/* command line options */
+static int sflg = 0;		/* -s - set shell from command line  */
+#ifdef USE_PAM
+static pam_handle_t *pamh = NULL;
+#endif
 
 /* external identifiers */
 
@@ -71,18 +73,21 @@ static void usage (void);
 static void new_fields (void);
 static int check_shell (const char *);
 static int restricted_shell (const char *);
+static void process_flags (int argc, char **argv);
+static void check_perms (const struct passwd *pw);
+static void update_shell (const char *user, char *loginsh);
 
 /*
  * usage - print command line syntax and exit
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: chsh [options] [LOGIN]\n"
-			   "\n"
-			   "Options:\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -s, --shell SHELL             new login shell for the user account\n"
-			   "\n"));
+	fputs (_("Usage: chsh [options] [LOGIN]\n"
+	         "\n"
+	         "Options:\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -s, --shell SHELL             new login shell for the user account\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
@@ -94,7 +99,7 @@ static void usage (void)
  */
 static void new_fields (void)
 {
-	printf (_("Enter the new value, or press ENTER for the default\n"));
+	puts (_("Enter the new value, or press ENTER for the default"));
 	change_field (loginsh, sizeof loginsh, _("Login Shell"));
 }
 
@@ -137,8 +142,9 @@ static int check_shell (const char *sh)
 #ifdef HAVE_GETUSERSHELL
 	setusershell ();
 	while ((cp = getusershell ())) {
-		if (*cp == '#')
+		if (*cp == '#') {
 			continue;
+		}
 
 		if (strcmp (cp, sh) == 0) {
 			found = 1;
@@ -147,15 +153,20 @@ static int check_shell (const char *sh)
 	}
 	endusershell ();
 #else
-	if ((fp = fopen (SHELLS_FILE, "r")) == (FILE *) 0)
+	fp = fopen (SHELLS_FILE, "r");
+	if (NULL == fp) {
 		return 0;
+	}
 
 	while (fgets (buf, sizeof (buf), fp)) {
-		if ((cp = strrchr (buf, '\n')))
+		cp = strrchr (buf, '\n');
+		if (NULL != cp) {
 			*cp = '\0';
+		}
 
-		if (buf[0] == '#')
+		if (buf[0] == '#') {
 			continue;
+		}
 
 		if (strcmp (buf, sh) == 0) {
 			found = 1;
@@ -168,69 +179,33 @@ static int check_shell (const char *sh)
 }
 
 /*
- * chsh - this command controls changes to the user's shell
+ *  * process_flags - parse the command line options
  *
- *	The only supported option is -s which permits the the login shell to
- *	be set from the command line.
+ *	It will not return if an error is encountered.
  */
-int main (int argc, char **argv)
+static void process_flags (int argc, char **argv)
 {
-	char *user;		/* User name                         */
-	int sflg = 0;		/* -s - set shell from command line  */
-	const struct passwd *pw;	/* Password entry from /etc/passwd   */
-	struct passwd pwent;	/* New password entry                */
+	int option_index = 0;
+	int c;
+	static struct option long_options[] = {
+		{"help", no_argument, NULL, 'h'},
+		{"shell", required_argument, NULL, 's'},
+		{NULL, 0, NULL, '\0'}
+	};
 
-#ifdef USE_PAM
-	pam_handle_t *pamh = NULL;
-	int retval;
-#endif
-
-	sanitize_env ();
-
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
-
-	/*
-	 * This command behaves different for root and non-root users.
-	 */
-	amroot = getuid () == 0;
-
-	/*
-	 * Get the program name. The program name is used as a prefix to
-	 * most error messages.
-	 */
-	Prog = Basename (argv[0]);
-
-	OPENLOG ("chsh");
-
-	/*
-	 * There is only one option, but use getopt() anyway to
-	 * keep things consistent.
-	 */
-	{
-		int option_index = 0;
-		int c;
-		static struct option long_options[] = {
-			{"help", no_argument, NULL, 'h'},
-			{"shell", required_argument, NULL, 's'},
-			{NULL, 0, NULL, '\0'}
-		};
-
-		while ((c =
-			getopt_long (argc, argv, "hs:", long_options,
-				     &option_index)) != -1) {
-			switch (c) {
-			case 'h':
-				usage ();
-				break;
-			case 's':
-				sflg++;
-				STRFCPY (loginsh, optarg);
-				break;
-			default:
-				usage ();
-			}
+	while ((c =
+		getopt_long (argc, argv, "hs:", long_options,
+		             &option_index)) != -1) {
+		switch (c) {
+		case 'h':
+			usage ();
+			break;
+		case 's':
+			sflg++;
+			STRFCPY (loginsh, optarg);
+			break;
+		default:
+			usage ();
 		}
 	}
 
@@ -238,54 +213,26 @@ int main (int argc, char **argv)
 	 * There should be only one remaining argument at most and it should
 	 * be the user's name.
 	 */
-	if (argc > optind + 1)
+	if (argc > (optind + 1)) {
 		usage ();
-
-	/*
-	 * Get the name of the user to check. It is either the command line
-	 * name, or the name getlogin() returns.
-	 */
-	if (optind < argc) {
-		user = argv[optind];
-		pw = xgetpwnam (user);
-		if (!pw) {
-			fprintf (stderr,
-				 _("%s: unknown user %s\n"), Prog, user);
-			exit (1);
-		}
-	} else {
-		pw = get_my_pwent ();
-		if (!pw) {
-			fprintf (stderr,
-				 _
-				 ("%s: Cannot determine your user name.\n"),
-				 Prog);
-			exit (1);
-		}
-		user = xstrdup (pw->pw_name);
 	}
+}
 
-#ifdef	USE_NIS
-	/*
-	 * Now we make sure this is a LOCAL password entry for this user ...
-	 */
-	if (__ispwNIS ()) {
-		char *nis_domain;
-		char *nis_master;
-
-		fprintf (stderr,
-			 _("%s: cannot change user '%s' on NIS client.\n"),
-			 Prog, user);
-
-		if (!yp_get_default_domain (&nis_domain) &&
-		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
-			fprintf (stderr,
-				 _
-				 ("%s: '%s' is the NIS master for this client.\n"),
-				 Prog, nis_master);
-		}
-		exit (1);
-	}
+/*
+ * check_perms - check if the caller is allowed to add a group
+ *
+ *	Non-root users are only allowed to change their shell, if their current
+ *	shell is not a restricted shell.
+ *
+ *	Non-root users must be authenticated.
+ *
+ *	It will not return if the user is not allowed.
+ */
+static void check_perms (const struct passwd *pw)
+{
+#ifdef USE_PAM
+	int retval;
+	struct passwd *pampw;
 #endif
 
 	/*
@@ -293,10 +240,11 @@ int main (int argc, char **argv)
 	 * UID of the user matches the current real UID.
 	 */
 	if (!amroot && pw->pw_uid != getuid ()) {
-		SYSLOG ((LOG_WARN, "can't change shell for `%s'", user));
+		SYSLOG ((LOG_WARN, "can't change shell for `%s'", pw->pw_name));
 		closelog ();
 		fprintf (stderr,
-			 _("You may not change the shell for %s.\n"), user);
+		         _("You may not change the shell for %s.\n"),
+		         pw->pw_name);
 		exit (1);
 	}
 
@@ -305,10 +253,11 @@ int main (int argc, char **argv)
 	 * is not a restricted one.
 	 */
 	if (!amroot && restricted_shell (pw->pw_shell)) {
-		SYSLOG ((LOG_WARN, "can't change shell for `%s'", user));
+		SYSLOG ((LOG_WARN, "can't change shell for `%s'", pw->pw_name));
 		closelog ();
 		fprintf (stderr,
-			 _("You may not change the shell for %s.\n"), user);
+		         _("You may not change the shell for %s.\n"),
+		         pw->pw_name);
 		exit (1);
 	}
 #ifdef WITH_SELINUX
@@ -319,10 +268,11 @@ int main (int argc, char **argv)
 	if ((pw->pw_uid != getuid ())
 	    && (is_selinux_enabled () > 0)
 	    && (selinux_check_passwd_access (PASSWD__CHSH) != 0)) {
-		SYSLOG ((LOG_WARN, "can't change shell for `%s'", user));
+		SYSLOG ((LOG_WARN, "can't change shell for `%s'", pw->pw_name));
 		closelog ();
 		fprintf (stderr,
-			 _("You may not change the shell for %s.\n"), user);
+		         _("You may not change the shell for %s.\n"),
+		         pw->pw_name);
 		exit (1);
 	}
 #endif
@@ -334,23 +284,20 @@ int main (int argc, char **argv)
 	 * before any changes can be made. Idea from util-linux
 	 * chfn/chsh.  --marekm
 	 */
-	if (!amroot && getdef_bool ("CHSH_AUTH"))
+	if (!amroot && getdef_bool ("CHSH_AUTH")) {
 		passwd_check (pw->pw_name, pw->pw_passwd, "chsh");
+        }
 
 #else				/* !USE_PAM */
 	retval = PAM_SUCCESS;
 
-	{
-		struct passwd *pampw;
-		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
-		if (pampw == NULL) {
-			retval = PAM_USER_UNKNOWN;
-		}
+	pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+	if (pampw == NULL) {
+		retval = PAM_USER_UNKNOWN;
+	}
 
-		if (retval == PAM_SUCCESS) {
-			retval = pam_start ("chsh", pampw->pw_name,
-					    &conv, &pamh);
-		}
+	if (retval == PAM_SUCCESS) {
+		retval = pam_start ("chsh", pampw->pw_name, &conv, &pamh);
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -372,13 +319,193 @@ int main (int argc, char **argv)
 		exit (E_NOPERM);
 	}
 #endif				/* USE_PAM */
+}
+
+/*
+ * update_shell - update the user's shell in the passwd database
+ *
+ *	Commit the user's entry after changing her shell field.
+ *
+ *	It will not return in case of error.
+ */
+static void update_shell (const char *user, char *newshell)
+{
+	const struct passwd *pw;	/* Password entry from /etc/passwd   */
+	struct passwd pwent;		/* New password entry                */
+
+	/*
+	 * Before going any further, raise the ulimit to prevent
+	 * colliding into a lowered ulimit, and set the real UID
+	 * to root to protect against unexpected signals. Any
+	 * keyboard signals are set to be ignored.
+	 */
+	if (setuid (0) != 0) {
+		SYSLOG ((LOG_ERR, "can't setuid(0)"));
+		closelog ();
+		fputs (_("Cannot change ID to root.\n"), stderr);
+		exit (1);
+	}
+	pwd_init ();
+
+	/*
+	 * The passwd entry is now ready to be committed back to
+	 * the password file. Get a lock on the file and open it.
+	 */
+	if (pw_lock () == 0) {
+		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
+		closelog ();
+		fputs (_("Cannot lock the password file; try again later.\n"),
+		       stderr);
+		exit (1);
+	}
+	if (pw_open (O_RDWR) == 0) {
+		SYSLOG ((LOG_ERR, "can't open /etc/passwd"));
+		closelog ();
+		fputs (_("Cannot open the password file.\n"), stderr);
+		pw_unlock ();
+		exit (1);
+	}
+
+	/*
+	 * Get the entry to update using pw_locate() - we want the real
+	 * one from /etc/passwd, not the one from getpwnam() which could
+	 * contain the shadow password if (despite the warnings) someone
+	 * enables AUTOSHADOW (or SHADOW_COMPAT in libc).  --marekm
+	 */
+	pw = pw_locate (user);
+	if (NULL == pw) {
+		pw_unlock ();
+		fprintf (stderr,
+		         _("%s: %s not found in /etc/passwd\n"), Prog, user);
+		exit (1);
+	}
+
+	/*
+	 * Make a copy of the entry, then change the shell field. The other
+	 * fields remain unchanged.
+	 */
+	pwent = *pw;
+	pwent.pw_shell = newshell;
+
+	/*
+	 * Update the passwd file entry. If there is a DBM file, update
+	 * that entry as well.
+	 */
+	if (pw_update (&pwent) == 0) {
+		SYSLOG ((LOG_ERR, "error updating passwd entry"));
+		closelog ();
+		fputs (_("Error updating the password entry.\n"), stderr);
+		pw_unlock ();
+		exit (1);
+	}
+
+	/*
+	 * Changes have all been made, so commit them and unlock the file.
+	 */
+	if (pw_close () == 0) {
+		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
+		closelog ();
+		fputs (_("Cannot commit password file changes.\n"), stderr);
+		pw_unlock ();
+		exit (1);
+	}
+	if (pw_unlock () == 0) {
+		SYSLOG ((LOG_ERR, "can't unlock /etc/passwd"));
+		closelog ();
+		fputs (_("Cannot unlock the password file.\n"), stderr);
+		exit (1);
+	}
+}
+
+/*
+ * chsh - this command controls changes to the user's shell
+ *
+ *	The only supported option is -s which permits the the login shell to
+ *	be set from the command line.
+ */
+int main (int argc, char **argv)
+{
+	char *user;		/* User name                         */
+	const struct passwd *pw;	/* Password entry from /etc/passwd   */
+
+	sanitize_env ();
+
+	setlocale (LC_ALL, "");
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+
+	/*
+	 * This command behaves different for root and non-root users.
+	 */
+	amroot = getuid () == 0;
+
+	/*
+	 * Get the program name. The program name is used as a prefix to
+	 * most error messages.
+	 */
+	Prog = Basename (argv[0]);
+
+	OPENLOG ("chsh");
+
+	/* parse the command line options */
+	process_flags (argc, argv);
+
+	/*
+	 * Get the name of the user to check. It is either the command line
+	 * name, or the name getlogin() returns.
+	 */
+	if (optind < argc) {
+		user = argv[optind];
+		pw = xgetpwnam (user);
+		if (!pw) {
+			fprintf (stderr,
+			         _("%s: unknown user %s\n"), Prog, user);
+			exit (1);
+		}
+	} else {
+		pw = get_my_pwent ();
+		if (!pw) {
+			fprintf (stderr,
+			         _
+			         ("%s: Cannot determine your user name.\n"),
+			         Prog);
+			exit (1);
+		}
+		user = xstrdup (pw->pw_name);
+	}
+
+#ifdef	USE_NIS
+	/*
+	 * Now we make sure this is a LOCAL password entry for this user ...
+	 */
+	if (__ispwNIS ()) {
+		char *nis_domain;
+		char *nis_master;
+
+		fprintf (stderr,
+		         _("%s: cannot change user '%s' on NIS client.\n"),
+		         Prog, user);
+
+		if (!yp_get_default_domain (&nis_domain) &&
+		    !yp_master (nis_domain, "passwd.byname", &nis_master)) {
+			fprintf (stderr,
+			         _
+			         ("%s: '%s' is the NIS master for this client.\n"),
+			         Prog, nis_master);
+		}
+		exit (1);
+	}
+#endif
+
+	check_perms (pw);
 
 	/*
 	 * Now get the login shell. Either get it from the password
 	 * file, or use the value from the command line.
 	 */
-	if (!sflg)
+	if (!sflg) {
 		STRFCPY (loginsh, pw->pw_shell);
+	}
 
 	/*
 	 * If the login shell was not set on the command line, let the user
@@ -406,98 +533,17 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 
-	/*
-	 * Before going any further, raise the ulimit to prevent
-	 * colliding into a lowered ulimit, and set the real UID
-	 * to root to protect against unexpected signals. Any
-	 * keyboard signals are set to be ignored.
-	 */
-	if (setuid (0)) {
-		SYSLOG ((LOG_ERR, "can't setuid(0)"));
-		closelog ();
-		fprintf (stderr, _("Cannot change ID to root.\n"));
-		exit (1);
-	}
-	pwd_init ();
+	update_shell (user, loginsh);
 
-	/*
-	 * The passwd entry is now ready to be committed back to
-	 * the password file. Get a lock on the file and open it.
-	 */
-	if (!pw_lock ()) {
-		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
-		closelog ();
-		fprintf (stderr,
-			 _
-			 ("Cannot lock the password file; try again later.\n"));
-		exit (1);
-	}
-	if (!pw_open (O_RDWR)) {
-		SYSLOG ((LOG_ERR, "can't open /etc/passwd"));
-		closelog ();
-		fprintf (stderr, _("Cannot open the password file.\n"));
-		pw_unlock ();
-		exit (1);
-	}
-
-	/*
-	 * Get the entry to update using pw_locate() - we want the real
-	 * one from /etc/passwd, not the one from getpwnam() which could
-	 * contain the shadow password if (despite the warnings) someone
-	 * enables AUTOSHADOW (or SHADOW_COMPAT in libc).  --marekm
-	 */
-	pw = pw_locate (user);
-	if (!pw) {
-		pw_unlock ();
-		fprintf (stderr,
-			 _("%s: %s not found in /etc/passwd\n"), Prog, user);
-		exit (1);
-	}
-
-	/*
-	 * Make a copy of the entry, then change the shell field. The other
-	 * fields remain unchanged.
-	 */
-	pwent = *pw;
-	pwent.pw_shell = loginsh;
-
-	/*
-	 * Update the passwd file entry. If there is a DBM file, update
-	 * that entry as well.
-	 */
-	if (!pw_update (&pwent)) {
-		SYSLOG ((LOG_ERR, "error updating passwd entry"));
-		closelog ();
-		fprintf (stderr, _("Error updating the password entry.\n"));
-		pw_unlock ();
-		exit (1);
-	}
-
-	/*
-	 * Changes have all been made, so commit them and unlock the file.
-	 */
-	if (!pw_close ()) {
-		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
-		closelog ();
-		fprintf (stderr, _("Cannot commit password file changes.\n"));
-		pw_unlock ();
-		exit (1);
-	}
-	if (!pw_unlock ()) {
-		SYSLOG ((LOG_ERR, "can't unlock /etc/passwd"));
-		closelog ();
-		fprintf (stderr, _("Cannot unlock the password file.\n"));
-		exit (1);
-	}
 	SYSLOG ((LOG_INFO, "changed user `%s' shell to `%s'", user, loginsh));
 
 	nscd_flush_cache ("passwd");
 
 #ifdef USE_PAM
-	if (retval == PAM_SUCCESS)
-		pam_end (pamh, PAM_SUCCESS);
+	pam_end (pamh, PAM_SUCCESS);
 #endif				/* USE_PAM */
 
 	closelog ();
 	exit (E_SUCCESS);
 }
+
