@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: expiry.c 1397 2007-11-18 23:15:26Z nekral-guest $"
+#ident "$Id: expiry.c 1742 2008-01-24 20:42:12Z nekral-guest $"
 
 #include <pwd.h>
 #include <signal.h>
@@ -44,7 +44,7 @@ static void usage (void);
 /*
  * catch_signals - signal catcher
  */
-static RETSIGTYPE catch_signals (int sig)
+static RETSIGTYPE catch_signals (unused int sig)
 {
 	exit (10);
 }
@@ -54,7 +54,7 @@ static RETSIGTYPE catch_signals (int sig)
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: expiry {-f|-c}\n"));
+	fputs (_("Usage: expiry {-f|-c}\n"), stderr);
 	exit (10);
 }
 
