@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: faillog.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: faillog.c 1903 2008-03-17 23:05:59Z nekral-guest $"
 
 #include <getopt.h>
 #include <pwd.h>
@@ -60,19 +60,19 @@ static struct stat statbuf;	/* fstat buffer for file size */
 
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: faillog [options]\n"
-			   "\n"
-			   "Options:\n"
-			   "  -a, --all                     display faillog records for all users\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -l, --lock-time SEC           after failed login lock accout to SEC seconds\n"
-			   "  -m, --maximum MAX             set maximum failed login counters to MAX\n"
-			   "  -r, --reset                   reset the counters of login failures\n"
-			   "  -t, --time DAYS               display faillog records more recent than DAYS\n"
-			   "  -u, --user LOGIN              display faillog record or maintains failure\n"
-			   "                                counters and limits (if used with -r, -m or -l\n"
-			   "                                options) only for user with LOGIN\n"
-			   "\n"));
+	fputs (_("Usage: faillog [options]\n"
+	         "\n"
+	         "Options:\n"
+	         "  -a, --all                     display faillog records for all users\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -l, --lock-time SEC           after failed login lock accout to SEC seconds\n"
+	         "  -m, --maximum MAX             set maximum failed login counters to MAX\n"
+	         "  -r, --reset                   reset the counters of login failures\n"
+	         "  -t, --time DAYS               display faillog records more recent than DAYS\n"
+	         "  -u, --user LOGIN              display faillog record or maintains failure\n"
+	         "                                counters and limits (if used with -r, -m or -l\n"
+	         "                                options) only for user with LOGIN\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
@@ -89,8 +89,7 @@ static void print_one (const struct faillog *fl, uid_t uid)
 #endif
 
 	if (!once) {
-		printf (_
-			("Login       Failures Maximum Latest                   On\n"));
+		puts (_("Login       Failures Maximum Latest                   On\n"));
 		once++;
 	}
 	pwent = getpwuid (uid); /* local, no need for xgetpwuid */
@@ -164,8 +163,14 @@ static void reset (void)
 
 	if (uflg)
 		reset_one (user);
-	else
-		for (uid = 0; reset_one (uid); uid++);
+	else {
+		struct passwd *pwent;
+
+		setpwent ();
+		while ( (pwent = getpwent ()) != NULL ) {
+			reset_one (pwent->pw_uid);
+		}
+	}
 }
 
 static void print (void)
