@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: gpasswd.c 1441 2007-11-22 21:55:12Z nekral-guest $"
+#ident "$Id: gpasswd.c 1901 2008-03-17 23:02:23Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -43,31 +43,68 @@
 #include "groupio.h"
 #include "nscd.h"
 #include "prototypes.h"
-#ifdef	SHADOWGRP
+#ifdef SHADOWGRP
 #include "sgroupio.h"
 #endif
 /*
  * Global variables
  */
+/* The name of this command, as it is invoked */
 static char *Prog;
 
 #ifdef SHADOWGRP
+/* Indicate if shadow groups are enabled on the system
+ * (/etc/gshadow present) */
 static int is_shadowgrp;
+static int gshadow_locked = 0;
 #endif
+static int group_locked = 0;
 
+/* Flags set by options */
 static int
  aflg = 0, Aflg = 0, dflg = 0, Mflg = 0, rflg = 0, Rflg = 0;
+/* The name of the group that is being affected */
+static char *group = NULL;
+/* The name of the user being added (-a) or removed (-d) from group */
+static char *user = NULL;
+/* The new list of members set with -M */
+static char *members = NULL;
+#ifdef SHADOWGRP
+/* The new list of group administrators set with -A */
+static char *admins = NULL;
+#endif
+/* The name of the caller */
+static char *myname = NULL;
+/* The UID of the caller */
+static unsigned long bywho = -1;
+/* Indicate if gpasswd was called by root */
+#define amroot	(0 == bywho)
 
-unsigned int bywho = -1;
-
+/* The number of retries for th user to provide and repeat a new password */
 #ifndef	RETRIES
 #define	RETRIES	3
 #endif
 
 /* local function prototypes */
 static void usage (void);
-static RETSIGTYPE catch_signals (int);
-static int check_list (const char *);
+static RETSIGTYPE catch_signals (int killed);
+static void fail_exit (int status);
+static int check_list (const char *users);
+static void process_flags (int argc, char **argv);
+static void check_flags (int argc, int opt_index);
+static void open_files (void);
+static void close_files (void);
+#ifdef SHADOWGRP
+static void get_group (struct group *gr, struct sgrp *sg);
+static void check_perms (const struct group *gr, const struct sgrp *sg);
+static void update_group (struct group *gr, struct sgrp *sg);
+static void change_passwd (struct group *gr, struct sgrp *sg);
+#else
+static void get_group (struct group *gr);
+static void check_perms (const struct group *gr);
+static void update_group (struct group *gr);
+static void change_passwd (struct group *gr);
+#endif
 
 /*
  * usage - display usage message
@@ -77,7 +114,7 @@ static void usage (void)
 	fprintf (stderr, _("Usage: %s [-r|-R] group\n"), Prog);
 	fprintf (stderr, _("       %s [-a user] group\n"), Prog);
 	fprintf (stderr, _("       %s [-d user] group\n"), Prog);
-#ifdef	SHADOWGRP
+#ifdef SHADOWGRP
 	fprintf (stderr,
 		 _("       %s [-A user,...] [-M user,...] group\n"), Prog);
 #else
@@ -98,23 +135,43 @@ static RETSIGTYPE catch_signals (int killed)
 {
 	static TERMIO sgtty;
 
-	if (killed)
+	if (killed) {
 		STTY (0, &sgtty);
-	else
+	} else {
 		GTTY (0, &sgtty);
+	}
 
 	if (killed) {
 		putchar ('\n');
 		fflush (stdout);
-		exit (killed);
+		fail_exit (killed);
 	}
 }
 
+/*
+ * fail_exit - undo as much as possible
+ */
+static void fail_exit (int status)
+{
+	if (group_locked) {
+		gr_unlock ();
+	}
+#ifdef SHADOWGRP
+	if (gshadow_locked) {
+		sgr_unlock ();
+	}
+#endif
+
+	exit (status);
+}
+
 /*
  * check_list - check a comma-separated list of user names for validity
  *
  *	check_list scans a comma-separated list of user names and checks
  *	that each listed name exists.
+ *
+ *	It returns 0 on success.
  */
 static int check_list (const char *users)
 {
@@ -123,16 +180,18 @@ static int check_list (const char *users)
 	int errors = 0;
 	size_t len;
 
-	for (start = users; start && *start; start = end) {
-		if ((end = strchr (start, ','))) {
+	for (start = users; (NULL != start) && ('\0' != *start); start = end) {
+		end = strchr (start, ',');
+		if (NULL != end) {
 			len = end - start;
 			end++;
 		} else {
 			len = strlen (start);
 		}
 
-		if (len > sizeof (username) - 1)
+		if (len > sizeof (username) - 1) {
 			len = sizeof (username) - 1;
+		}
 		strncpy (username, start, len);
 		username[len] = '\0';
 
@@ -140,9 +199,10 @@ static int check_list (const char *users)
 		 * This user must exist.
 		 */
 
-		if (!getpwnam (username)) { /* local, no need for xgetpwnam */
+		/* local, no need for xgetpwnam */
+		if (getpwnam (username) == NULL) {
 			fprintf (stderr, _("%s: unknown user %s\n"),
-				 Prog, username);
+			         Prog, username);
 			errors++;
 		}
 	}
@@ -152,73 +212,22 @@ static int check_list (const char *users)
 static void failure (void)
 {
 	fprintf (stderr, _("%s: Permission denied.\n"), Prog);
-	exit (1);
+	fail_exit (1);
 }
 
 /*
- * gpasswd - administer the /etc/group file
- *
- *	-a user		add user to the named group
- *	-d user		remove user from the named group
- *	-r		remove password from the named group
- *	-R		restrict access to the named group
- *	-A user,...	make list of users the administrative users
- *	-M user,...	make list of users the group members
+ * process_flags - process the command line options and arguments
  */
-int main (int argc, char **argv)
+static void process_flags (int argc, char **argv)
 {
 	int flag;
-	char *cp;
-	int amroot;
-	int retries;
-	struct group const*gr = NULL;
-	struct group grent;
-	static char pass[BUFSIZ];
 
-#ifdef	SHADOWGRP
-	struct sgrp const*sg = NULL;
-	struct sgrp sgent;
-	char *admins = NULL;
-#endif
-	struct passwd *pw = NULL;
-	char *myname;
-	char *user = NULL;
-	char *group = NULL;
-	char *members = NULL;
-
-#ifdef WITH_AUDIT
-	audit_help_open ();
-#endif
-
-	sanitize_env ();
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
-
-	/*
-	 * Make a note of whether or not this command was invoked by root.
-	 * This will be used to bypass certain checks later on. Also, set
-	 * the real user ID to match the effective user ID. This will
-	 * prevent the invoker from issuing signals which would interfer
-	 * with this command.
-	 */
-	amroot = getuid () == 0;
-	bywho = getuid ();
-	Prog = Basename (argv[0]);
-
-	OPENLOG ("gpasswd");
-	setbuf (stdout, NULL);
-	setbuf (stderr, NULL);
-
-#ifdef SHADOWGRP
-	is_shadowgrp = sgr_file_present ();
-#endif
 	while ((flag = getopt (argc, argv, "a:A:d:gM:rR")) != EOF) {
 		switch (flag) {
 		case 'a':	/* add a user */
 			user = optarg;
 			/* local, no need for xgetpwnam */
-			if (!getpwnam (user)) {
+			if (getpwnam (user) == NULL) {
 				fprintf (stderr,
 					 _("%s: unknown user %s\n"), Prog,
 					 user);
@@ -226,7 +235,7 @@ int main (int argc, char **argv)
 				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 					      "adding to group", user, -1, 0);
 #endif
-				exit (1);
+				fail_exit (1);
 			}
 			aflg++;
 			break;
@@ -245,11 +254,12 @@ int main (int argc, char **argv)
 					 _
 					 ("%s: shadow group passwords required for -A\n"),
 					 Prog);
-				exit (2);
+				fail_exit (2);
 			}
 			admins = optarg;
-			if (check_list (admins))
-				exit (1);
+			if (check_list (admins) != 0) {
+				fail_exit (1);
+			}
 			Aflg++;
 			break;
 #endif
@@ -269,8 +279,9 @@ int main (int argc, char **argv)
 				failure ();
 			}
 			members = optarg;
-			if (check_list (members))
-				exit (1);
+			if (check_list (members) != 0) {
+				fail_exit (1);
+			}
 			Mflg++;
 			break;
 		case 'r':	/* remove group password */
@@ -284,12 +295,468 @@ int main (int argc, char **argv)
 		}
 	}
 
+	/* Get the name of the group that is being affected. */
+	group = argv[optind];
+
+	check_flags (argc, optind);
+}
+
+/*
+ * check_flags - check the validity of options
+ */
+static void check_flags (int argc, int opt_index)
+{
 	/*
 	 * Make sure exclusive flags are exclusive
 	 */
-
-	if (aflg + dflg + rflg + Rflg + (Aflg || Mflg) > 1)
+	if (aflg + dflg + rflg + Rflg + (Aflg || Mflg) > 1) {
 		usage ();
+	}
+
+	/*
+	 * Make sure one (and only one) group was provided
+	 */
+	if ((argc != (opt_index+1)) || (NULL == group)) {
+		usage ();
+	}
+}
+
+/*
+ * open_files - lock and open the group databases
+ *
+ *	It will call exit in case of error.
+ */
+static void open_files (void)
+{
+	if (gr_lock () == 0) {
+		fprintf (stderr, _("%s: can't get lock\n"), Prog);
+		SYSLOG ((LOG_WARN, "failed to get lock for /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "locking /etc/group", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+	group_locked++;
+#ifdef SHADOWGRP
+	if (is_shadowgrp) {
+		if (sgr_lock () == 0) {
+			fprintf (stderr,
+			         _("%s: can't get shadow lock\n"), Prog);
+			SYSLOG ((LOG_WARN, "failed to get lock for /etc/gshadow"));
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			              "locking /etc/gshadow", group, -1, 0);
+#endif
+			fail_exit (1);
+		}
+		gshadow_locked++;
+	}
+#endif
+	if (gr_open (O_RDWR) == 0) {
+		fprintf (stderr, _("%s: can't open file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "opening /etc/group", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+#ifdef SHADOWGRP
+	if (is_shadowgrp && (sgr_open (O_RDWR) == 0)) {
+		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "opening /etc/gshadow", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+#endif
+}
+
+/*
+ * close_files - close and unlock the group databases
+ *
+ *	This cause any changes in the databases to be committed.
+ *
+ *	It will call exit in case of error.
+ */
+static void close_files (void)
+{
+	if (gr_close () == 0) {
+		fprintf (stderr, _("%s: can't re-write file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot re-write /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "rewriting /etc/group", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+#ifdef SHADOWGRP
+	if (is_shadowgrp && (sgr_close () == 0)) {
+		fprintf (stderr, _("%s: can't re-write shadow file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot re-write /etc/gshadow"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "rewriting /etc/gshadow", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+	if (is_shadowgrp) {
+		/* TODO: same logging as in open_files & for /etc/group */
+		sgr_unlock ();
+		gshadow_locked--;
+	}
+#endif
+	if (gr_unlock () == 0) {
+		fprintf (stderr, _("%s: can't unlock file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "unlocking group file", group, -1, 0);
+#endif
+		exit (1);
+	}
+	group_locked--;
+}
+
+/*
+ * check_perms - check if the user is allowed to change the password of
+ *               the specified group.
+ *
+ *	It only returns if the user is allowed.
+ */
+#ifdef SHADOWGRP
+static void check_perms (const struct group *gr, const struct sgrp *sg)
+#else
+static void check_perms (const struct group *gr)
+#endif
+{
+#ifdef SHADOWGRP
+	if (is_shadowgrp) {
+		/*
+		 * The policy here for changing a group is that
+		 * 1) you must be root or
+		 * 2) you must be listed as an administrative member.
+		 * Administrative members can do anything to a group that
+		 * the root user can.
+		 */
+		if (!amroot && !is_on_list (sg->sg_adm, myname)) {
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			              "modify group", group, -1, 0);
+#endif
+			failure ();
+		}
+	} else
+#endif				/* ! SHADOWGRP */
+	{
+#ifdef FIRST_MEMBER_IS_ADMIN
+		/*
+		 * The policy here for changing a group is that
+		 * 1) you must be root or
+		 * 2) you must be the first listed member of the group.
+		 * The first listed member of a group can do anything to
+		 * that group that the root user can. The rationale for
+		 * this hack is that the FIRST user is probably the most
+		 * important user in this entire group.
+		 *
+		 * This feature enabled by default could be a security
+		 * problem when installed on existing systems where the
+		 * first group member might be just a normal user.
+		 * --marekm
+		 */
+		if (!amroot) {
+			if (gr->gr_mem[0] == (char *) 0) {
+#ifdef WITH_AUDIT
+				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				              "modifying group", group, -1, 0);
+#endif
+				failure ();
+			}
+
+			if (strcmp (gr->gr_mem[0], myname) != 0) {
+#ifdef WITH_AUDIT
+				audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				              "modifying group", myname, -1, 0);
+#endif
+				failure ();
+			}
+		}
+#else				/* ! FIRST_MEMBER_IS_ADMIN */
+		if (!amroot) {
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			              "modifying group", group, -1, 0);
+#endif
+			failure ();
+		}
+#endif
+	}
+}
+
+/*
+ * update_group - Update the group information in the databases
+ */
+#ifdef SHADOWGRP
+static void update_group (struct group *gr, struct sgrp *sg)
+#else
+static void update_group (struct group *gr)
+#endif
+{
+	if (gr_update (gr) == 0) {
+		fprintf (stderr, _("%s: can't update entry\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot update /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "updating /etc/group", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+#ifdef SHADOWGRP
+	if (is_shadowgrp && (sgr_update (sg) == 0)) {
+		fprintf (stderr, _("%s: can't update shadow entry\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot update /etc/gshadow"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "updating /etc/gshadow", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+#endif
+}
+
+/*
+ * get_group - get the current information for the group
+ *
+ *	The information are copied in group structure(s) so that they can be
+ *	modified later.
+ *
+ *	Note: If !is_shadowgrp, *sg will not be initialized.
+ */
+#ifdef SHADOWGRP
+static void get_group (struct group *gr, struct sgrp *sg)
+#else
+static void get_group (struct group *gr)
+#endif
+{
+	struct group const*tmpgr = NULL;
+	struct sgrp const*tmpsg = NULL;
+
+	if (gr_open (O_RDONLY) == 0) {
+		fprintf (stderr, _("%s: can't open file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "opening /etc/group", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+
+	tmpgr = gr_locate (group);
+	if (NULL == tmpgr) {
+		fprintf (stderr, _("unknown group: %s\n"), group);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "group lookup", group, -1, 0);
+#endif
+		failure ();
+	}
+
+	*gr = *tmpgr;
+	gr->gr_name = xstrdup (tmpgr->gr_name);
+	gr->gr_passwd = xstrdup (tmpgr->gr_passwd);
+	gr->gr_mem = dup_list (tmpgr->gr_mem);
+
+	if (gr_close () == 0) {
+		fprintf (stderr, _("%s: can't close file\n"), Prog);
+		SYSLOG ((LOG_WARN, "cannot close /etc/group"));
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "closing /etc/group", group, -1, 0);
+#endif
+		fail_exit (1);
+	}
+
+#ifdef SHADOWGRP
+	if (is_shadowgrp) {
+		if (sgr_open (O_RDONLY) == 0) {
+			fprintf (stderr,
+			         _("%s: can't open shadow file\n"), Prog);
+			SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			              "opening /etc/gshadow", group, -1, 0);
+#endif
+			fail_exit (1);
+		}
+		tmpsg = sgr_locate (group);
+		if (NULL != tmpsg) {
+			*sg = *tmpsg;
+			sg->sg_name = xstrdup (tmpsg->sg_name);
+			sg->sg_passwd = xstrdup (tmpsg->sg_passwd);
+
+			sg->sg_mem = dup_list (tmpsg->sg_mem);
+			sg->sg_adm = dup_list (tmpsg->sg_adm);
+		} else {
+			sg->sg_name = xstrdup (group);
+			sg->sg_passwd = gr->gr_passwd;
+			gr->gr_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
+
+			sg->sg_mem = dup_list (gr->gr_mem);
+
+			sg->sg_adm = (char **) xmalloc (sizeof (char *) * 2);
+#ifdef FIRST_MEMBER_IS_ADMIN
+			if (sg->sg_mem[0]) {
+				sg->sg_adm[0] = xstrdup (sg->sg_mem[0]);
+				sg->sg_adm[1] = NULL;
+			} else
+#endif
+			{
+				sg->sg_adm[0] = NULL;
+			}
+
+		}
+		if (sgr_close () == 0) {
+			fprintf (stderr,
+			         _("%s: can't close shadow file\n"), Prog);
+			SYSLOG ((LOG_WARN, "cannot close /etc/gshadow"));
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			              "closing /etc/gshadow", group, -1, 0);
+#endif
+			fail_exit (1);
+		}
+	}
+#endif				/* SHADOWGRP */
+}
+
+/*
+ * change_passwd - change the group's password
+ *
+ *	Get the new password from the user and update the password in the
+ *	group's structure.
+ *
+ *	It will call exit in case of error.
+ */
+#ifdef SHADOWGRP
+static void change_passwd (struct group *gr, struct sgrp *sg)
+#else
+static void change_passwd (struct group *gr)
+#endif
+{
+	char *cp;
+	static char pass[BUFSIZ];
+	int retries;
+
+	/*
+	 * A new password is to be entered and it must be encrypted, etc.
+	 * The password will be prompted for twice, and both entries must be
+	 * identical. There is no need to validate the old password since
+	 * the invoker is either the group owner, or root.
+	 */
+	printf (_("Changing the password for group %s\n"), group);
+
+	for (retries = 0; retries < RETRIES; retries++) {
+		cp = getpass (_("New Password: "));
+		if (NULL == cp) {
+			fail_exit (1);
+		}
+
+		STRFCPY (pass, cp);
+		strzero (cp);
+		cp = getpass (_("Re-enter new password: "));
+		if (NULL == cp) {
+			fail_exit (1);
+		}
+
+		if (strcmp (pass, cp) == 0) {
+			strzero (cp);
+			break;
+		}
+
+		strzero (cp);
+		memzero (pass, sizeof pass);
+
+		if (retries + 1 < RETRIES) {
+			puts (_("They don't match; try again"));
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			              "changing password", group, -1, 0);
+#endif
+		}
+	}
+
+	if (retries == RETRIES) {
+		fprintf (stderr, _("%s: Try again later\n"), Prog);
+		fail_exit (1);
+	}
+
+	cp = pw_encrypt (pass, crypt_make_salt (NULL, NULL));
+	memzero (pass, sizeof pass);
+#ifdef SHADOWGRP
+	if (is_shadowgrp) {
+		sg->sg_passwd = cp;
+	} else
+#endif
+	{
+		gr->gr_passwd = cp;
+	}
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+	              "changing password", group, -1, 1);
+#endif
+	SYSLOG ((LOG_INFO, "change the password for group %s by %s", group,
+	        myname));
+}
+
+/*
+ * gpasswd - administer the /etc/group file
+ *
+ *	-a user		add user to the named group
+ *	-d user		remove user from the named group
+ *	-r		remove password from the named group
+ *	-R		restrict access to the named group
+ *	-A user,...	make list of users the administrative users
+ *	-M user,...	make list of users the group members
+ */
+int main (int argc, char **argv)
+{
+	struct group grent;
+#ifdef SHADOWGRP
+	struct sgrp sgent;
+#endif
+	struct passwd *pw = NULL;
+
+#ifdef WITH_AUDIT
+	audit_help_open ();
+#endif
+
+	sanitize_env ();
+	setlocale (LC_ALL, "");
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+
+	/*
+	 * Make a note of whether or not this command was invoked by root.
+	 * This will be used to bypass certain checks later on. Also, set
+	 * the real user ID to match the effective user ID. This will
+	 * prevent the invoker from issuing signals which would interfer
+	 * with this command.
+	 */
+	bywho = getuid ();
+	Prog = Basename (argv[0]);
+
+	OPENLOG ("gpasswd");
+	setbuf (stdout, NULL);
+	setbuf (stderr, NULL);
+
+#ifdef SHADOWGRP
+	is_shadowgrp = sgr_file_present ();
+#endif
+
+	/* Parse the options */
+	process_flags (argc, argv);
 
 	/*
 	 * Determine the name of the user that invoked this command. This
@@ -299,157 +766,33 @@ int main (int argc, char **argv)
 	 */
 
 	pw = get_my_pwent ();
-	if (!pw) {
-		fprintf (stderr, _("Who are you?\n"));
+	if (NULL == pw) {
+		fputs (_("Who are you?\n"), stderr);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "user lookup", NULL,
-			      bywho, 0);
+		              bywho, 0);
 #endif
 		failure ();
 	}
 	myname = xstrdup (pw->pw_name);
 
 	/*
-	 * Get the name of the group that is being affected. The group entry
-	 * will be completely replicated so it may be modified later on.
+	 * Replicate the group so it can be modified later on.
 	 */
-
-	if (!(group = argv[optind]))
-		usage ();
-
-	if (!gr_open (O_RDONLY)) {
-		fprintf (stderr, _("%s: can't open file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "opening /etc/group",
-			      group, -1, 0);
-#endif
-		exit (1);
-	}
-
-	if (!(gr = gr_locate (group))) {
-		fprintf (stderr, _("unknown group: %s\n"), group);
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "group lookup", group,
-			      -1, 0);
-#endif
-		failure ();
-	}
-	grent = *gr;
-	grent.gr_name = xstrdup (gr->gr_name);
-	grent.gr_passwd = xstrdup (gr->gr_passwd);
-
-	grent.gr_mem = dup_list (gr->gr_mem);
-	if (!gr_close ()) {
-		fprintf (stderr, _("%s: can't close file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot close /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "closing /etc/group", group, -1, 0);
-#endif
-		exit (1);
-	}
-#ifdef	SHADOWGRP
-	if (!sgr_open (O_RDONLY)) {
-		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "opening /etc/gshadow", group, -1, 0);
-#endif
-		exit (1);
-	}
-	if ((sg = sgr_locate (group))) {
-		sgent = *sg;
-		sgent.sg_name = xstrdup (sg->sg_name);
-		sgent.sg_passwd = xstrdup (sg->sg_passwd);
-
-		sgent.sg_mem = dup_list (sg->sg_mem);
-		sgent.sg_adm = dup_list (sg->sg_adm);
-	} else {
-		sgent.sg_name = xstrdup (group);
-		sgent.sg_passwd = grent.gr_passwd;
-		grent.gr_passwd = "!";	/* XXX warning: const */
-
-		sgent.sg_mem = dup_list (grent.gr_mem);
-
-		sgent.sg_adm = (char **) xmalloc (sizeof (char *) * 2);
-#ifdef FIRST_MEMBER_IS_ADMIN
-		if (sgent.sg_mem[0]) {
-			sgent.sg_adm[0] = xstrdup (sgent.sg_mem[0]);
-			sgent.sg_adm[1] = 0;
-		} else
-#endif
-			sgent.sg_adm[0] = 0;
-
-		sg = &sgent;
-	}
-	if (!sgr_close ()) {
-		fprintf (stderr, _("%s: can't close shadow file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot close /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "closing /etc/gshadow", group, -1, 0);
-#endif
-		exit (1);
-	}
-
-	/*
-	 * The policy here for changing a group is that 1) you must be root
-	 * or 2). you must be listed as an administrative member. 
-	 * Administrative members can do anything to a group that the root
-	 * user can.
-	 */
-	if (!amroot && !is_on_list (sgent.sg_adm, myname)) {
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modify group", group,
-			      -1, 0);
-#endif
-		failure ();
-	}
-#else				/* ! SHADOWGRP */
-
-#ifdef FIRST_MEMBER_IS_ADMIN
-	/*
-	 * The policy here for changing a group is that 1) you must bes root
-	 * or 2) you must be the first listed member of the group. The
-	 * first listed member of a group can do anything to that group that
-	 * the root user can. The rationale for this hack is that the FIRST
-	 * user is probably the most important user in this entire group.
-	 */
-	if (!amroot) {
-		if (grent.gr_mem[0] == (char *) 0) {
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				      "modifying group", group, -1, 0);
-#endif
-			failure ();
-		}
-
-		if (strcmp (grent.gr_mem[0], myname) != 0) {
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				      "modifying group", myname, -1, 0);
-#endif
-			failure ();
-		}
-	}
+#ifdef SHADOWGRP
+	get_group (&grent, &sgent);
 #else
-	/*
-	 * This feature enabled by default could be a security problem when
-	 * installed on existing systems where the first group member might
-	 * be just a normal user.  --marekm
-	 */
-	if (!amroot) {
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group",
-			      group, -1, 0);
-#endif
-		failure ();
-	}
+	get_group (&grent);
 #endif
 
-#endif				/* SHADOWGRP */
+	/*
+	 * Check if the user is allowed to change the password of this group.
+	 */
+#ifdef SHADOWGRP
+	check_perms (&grent, &sgent);
+#else
+	check_perms (&grent);
+#endif
 
 	/*
 	 * Removing a password is straight forward. Just set the password
@@ -457,15 +800,15 @@ int main (int argc, char **argv)
 	 */
 	if (rflg) {
 		grent.gr_passwd = "";	/* XXX warning: const */
-#ifdef	SHADOWGRP
+#ifdef SHADOWGRP
 		sgent.sg_passwd = "";	/* XXX warning: const */
 #endif
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "deleting group password", group, -1, 1);
+		              "deleting group password", group, -1, 1);
 #endif
 		SYSLOG ((LOG_INFO, "remove password from group %s by %s",
-			 group, myname));
+		         group, myname));
 		goto output;
 	} else if (Rflg) {
 		/*
@@ -473,12 +816,12 @@ int main (int argc, char **argv)
 		 * field to "!".
 		 */
 		grent.gr_passwd = "!";	/* XXX warning: const */
-#ifdef	SHADOWGRP
+#ifdef SHADOWGRP
 		sgent.sg_passwd = "!";	/* XXX warning: const */
 #endif
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "restrict access to group", group, -1, 1);
+		              "restrict access to group", group, -1, 1);
 #endif
 		SYSLOG ((LOG_INFO, "restrict access to group %s by %s",
 			 group, myname));
@@ -492,15 +835,17 @@ int main (int argc, char **argv)
 	if (aflg) {
 		printf (_("Adding user %s to group %s\n"), user, group);
 		grent.gr_mem = add_list (grent.gr_mem, user);
-#ifdef	SHADOWGRP
-		sgent.sg_mem = add_list (sgent.sg_mem, user);
+#ifdef SHADOWGRP
+		if (is_shadowgrp) {
+			sgent.sg_mem = add_list (sgent.sg_mem, user);
+		}
 #endif
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group member",
-			      user, -1, 1);
+		              user, -1, 1);
 #endif
 		SYSLOG ((LOG_INFO, "add member %s to group %s by %s", user,
-			 group, myname));
+		         group, myname));
 		goto output;
 	}
 
@@ -517,30 +862,32 @@ int main (int argc, char **argv)
 			removed = 1;
 			grent.gr_mem = del_list (grent.gr_mem, user);
 		}
-#ifdef	SHADOWGRP
-		if (is_on_list (sgent.sg_mem, user)) {
-			removed = 1;
-			sgent.sg_mem = del_list (sgent.sg_mem, user);
+#ifdef SHADOWGRP
+		if (is_shadowgrp) {
+			if (is_on_list (sgent.sg_mem, user)) {
+				removed = 1;
+				sgent.sg_mem = del_list (sgent.sg_mem, user);
+			}
 		}
 #endif
 		if (!removed) {
 			fprintf (stderr, _("%s: unknown member %s\n"),
-				 Prog, user);
+			         Prog, user);
 #ifdef WITH_AUDIT
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				      "deleting member", user, -1, 0);
+			              "deleting member", user, -1, 0);
 #endif
-			exit (1);
+			fail_exit (1);
 		}
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting member",
-			      user, -1, 1);
+		              user, -1, 1);
 #endif
 		SYSLOG ((LOG_INFO, "remove member %s from group %s by %s",
-			 user, group, myname));
+		         user, group, myname));
 		goto output;
 	}
-#ifdef	SHADOWGRP
+#ifdef SHADOWGRP
 	/*
 	 * Replacing the entire list of administators is simple. Check the
 	 * list to make sure everyone is a real user. Then slap the new list
@@ -549,13 +896,14 @@ int main (int argc, char **argv)
 	if (Aflg) {
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "setting group admin",
-			      group, -1, 1);
+		              group, -1, 1);
 #endif
 		SYSLOG ((LOG_INFO, "set administrators of %s to %s",
-			 group, admins));
+		         group, admins));
 		sgent.sg_adm = comma_to_list (admins);
-		if (!Mflg)
+		if (!Mflg) {
 			goto output;
+		}
 	}
 #endif
 
@@ -567,7 +915,7 @@ int main (int argc, char **argv)
 	if (Mflg) {
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "setting group members", group, -1, 1);
+		              "setting group members", group, -1, 1);
 #endif
 		SYSLOG ((LOG_INFO, "set members of %s to %s", group, members));
 #ifdef SHADOWGRP
@@ -582,13 +930,13 @@ int main (int argc, char **argv)
 	 * be a tty. The typical keyboard signals are caught so the termio
 	 * modes can be restored.
 	 */
-	if (!isatty (0) || !isatty (1)) {
+	if ((isatty (0) == 0) || (isatty (1) == 0)) {
 		fprintf (stderr, _("%s: Not a tty\n"), Prog);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing password",
-			      group, -1, 0);
+		              group, -1, 0);
 #endif
-		exit (1);
+		fail_exit (1);
 	}
 
 	catch_signals (0);	/* save tty modes */
@@ -597,63 +945,16 @@ int main (int argc, char **argv)
 	signal (SIGINT, catch_signals);
 	signal (SIGQUIT, catch_signals);
 	signal (SIGTERM, catch_signals);
-#ifdef	SIGTSTP
+#ifdef SIGTSTP
 	signal (SIGTSTP, catch_signals);
 #endif
 
-	/*
-	 * A new password is to be entered and it must be encrypted, etc.
-	 * The password will be prompted for twice, and both entries must be
-	 * identical. There is no need to validate the old password since
-	 * the invoker is either the group owner, or root.
-	 */
-	printf (_("Changing the password for group %s\n"), group);
-
-	for (retries = 0; retries < RETRIES; retries++) {
-		if (!(cp = getpass (_("New Password: "))))
-			exit (1);
-
-		STRFCPY (pass, cp);
-		strzero (cp);
-		if (!(cp = getpass (_("Re-enter new password: "))))
-			exit (1);
-
-		if (strcmp (pass, cp) == 0) {
-			strzero (cp);
-			break;
-		}
-
-		strzero (cp);
-		memzero (pass, sizeof pass);
-
-		if (retries + 1 < RETRIES) {
-			puts (_("They don't match; try again"));
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				      "changing password", group, -1, 0);
-#endif
-		}
-	}
-
-	if (retries == RETRIES) {
-		fprintf (stderr, _("%s: Try again later\n"), Prog);
-		exit (1);
-	}
-
-	cp = pw_encrypt (pass, crypt_make_salt (NULL, NULL));
-	memzero (pass, sizeof pass);
+	/* Prompt for the new password */
 #ifdef SHADOWGRP
-	if (is_shadowgrp)
-		sgent.sg_passwd = cp;
-	else
+	change_passwd (&grent, &sgent);
+#else
+	change_passwd (&grent);
 #endif
-		grent.gr_passwd = cp;
-#ifdef WITH_AUDIT
-	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing password", group,
-		      -1, 1);
-#endif
-	SYSLOG ((LOG_INFO, "change the password for group %s by %s", group,
-		 myname));
 
 	/*
 	 * This is the common arrival point to output the new group file.
@@ -662,108 +963,27 @@ int main (int argc, char **argv)
 	 * output, etc.
 	 */
       output:
-	if (setuid (0)) {
-		fprintf (stderr, _("Cannot change ID to root.\n"));
+	if (setuid (0) != 0) {
+		fputs (_("Cannot change ID to root.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing id to root",
-			      group, -1, 0);
+		              group, -1, 0);
 #endif
 		closelog ();
-		exit (1);
+		fail_exit (1);
 	}
 	pwd_init ();
 
-	if (!gr_lock ()) {
-		fprintf (stderr, _("%s: can't get lock\n"), Prog);
-		SYSLOG ((LOG_WARN, "failed to get lock for /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "locking /etc/group",
-			      group, -1, 0);
+	open_files ();
+
+#ifdef SHADOWGRP
+	update_group (&grent, &sgent);
+#else
+	update_group (&grent);
 #endif
-		exit (1);
-	}
-#ifdef	SHADOWGRP
-	if (is_shadowgrp && !sgr_lock ()) {
-		fprintf (stderr, _("%s: can't get shadow lock\n"), Prog);
-		SYSLOG ((LOG_WARN, "failed to get lock for /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "locking /etc/gshadow", group, -1, 0);
-#endif
-		exit (1);
-	}
-#endif
-	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: can't open file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "opening /etc/group",
-			      group, -1, 0);
-#endif
-		exit (1);
-	}
-#ifdef	SHADOWGRP
-	if (is_shadowgrp && !sgr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: can't open shadow file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot open /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "opening /etc/gshadow", group, -1, 0);
-#endif
-		exit (1);
-	}
-#endif
-	if (!gr_update (&grent)) {
-		fprintf (stderr, _("%s: can't update entry\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot update /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "updating /etc/group",
-			      group, -1, 0);
-#endif
-		exit (1);
-	}
-#ifdef	SHADOWGRP
-	if (is_shadowgrp && !sgr_update (&sgent)) {
-		fprintf (stderr, _("%s: can't update shadow entry\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot update /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "updating /etc/gshadow", group, -1, 0);
-#endif
-		exit (1);
-	}
-#endif
-	if (!gr_close ()) {
-		fprintf (stderr, _("%s: can't re-write file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot re-write /etc/group"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "rewriting /etc/group", group, -1, 0);
-#endif
-		exit (1);
-	}
-#ifdef	SHADOWGRP
-	if (is_shadowgrp && !sgr_close ()) {
-		fprintf (stderr, _("%s: can't re-write shadow file\n"), Prog);
-		SYSLOG ((LOG_WARN, "cannot re-write /etc/gshadow"));
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "rewriting /etc/gshadow", group, -1, 0);
-#endif
-		exit (1);
-	}
-	if (is_shadowgrp)
-		sgr_unlock ();
-#endif
-	if (!gr_unlock ()) {
-		fprintf (stderr, _("%s: can't unlock file\n"), Prog);
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-			      "unlocking group file", group, -1, 0);
-#endif
-		exit (1);
-	}
+
+	close_files ();
 
 	nscd_flush_cache ("group");
 
