@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: groupdel.c,v 1.32 2006/08/15 17:25:58 kloczek Exp $"
+#ident "$Id: groupdel.c 1894 2008-03-08 23:52:50Z nekral-guest $"
 
 #include <ctype.h>
 #include <fcntl.h>
@@ -52,7 +52,6 @@
  */
 static char *group_name;
 static char *Prog;
-static int errors;
 static gid_t group_id = -1;
 
 #ifdef	SHADOWGRP
@@ -70,6 +69,7 @@ static int is_shadow_grp;
 
 /* local function prototypes */
 static void usage (void);
+static void fail_exit (int);
 static void grp_update (void);
 static void close_files (void);
 static void open_files (void);
@@ -80,10 +80,30 @@ static void group_busy (gid_t);
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: groupdel group\n"));
+	fputs (_("Usage: groupdel group\n"), stderr);
 	exit (E_USAGE);
 }
 
+/*
+ * fail_exit - exit with a failure code after unlocking the files
+ */
+static void fail_exit (int code)
+{
+	(void) gr_unlock ();
+#ifdef	SHADOWGRP
+	if (is_shadow_grp) {
+		sgr_unlock ();
+	}
+#endif
+
+#ifdef	WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
+	              group_name, -1, 0);
+#endif
+
+	exit (code);
+}
+
 /*
  * grp_update - update group file entries
  *
@@ -93,23 +113,21 @@ static void grp_update (void)
 {
 	if (!gr_remove (group_name)) {
 		fprintf (stderr, _("%s: error removing group entry\n"), Prog);
-		errors++;
+		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
 	/*
 	 * Delete the shadow group entries as well.
 	 */
-	if (is_shadow_grp && !sgr_remove (group_name)) {
-		fprintf (stderr,
-			 _("%s: error removing shadow group entry\n"), Prog);
-		errors++;
+	if (is_shadow_grp && (sgr_locate (group_name) != NULL)) {
+		if (!sgr_remove (group_name)) {
+			fprintf (stderr,
+			         _("%s: error removing shadow group entry\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
 	}
 #endif				/* SHADOWGRP */
-#ifdef WITH_AUDIT
-	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group", group_name,
-		      group_id, 1);
-#endif
-	SYSLOG ((LOG_INFO, "remove group `%s'\n", group_name));
 	return;
 }
 
@@ -121,16 +139,22 @@ static void grp_update (void)
  */
 static void close_files (void)
 {
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group", group_name,
+	              group_id, 1);
+#endif
+	SYSLOG ((LOG_INFO, "remove group `%s'\n", group_name));
+
 	if (!gr_close ()) {
 		fprintf (stderr, _("%s: cannot rewrite group file\n"), Prog);
-		errors++;
+		fail_exit (E_GRP_UPDATE);
 	}
 	gr_unlock ();
 #ifdef	SHADOWGRP
 	if (is_shadow_grp && !sgr_close ()) {
 		fprintf (stderr,
 			 _("%s: cannot rewrite shadow group file\n"), Prog);
-		errors++;
+		fail_exit (E_GRP_UPDATE);
 	}
 	if (is_shadow_grp)
 		sgr_unlock ();
@@ -146,22 +170,22 @@ static void open_files (void)
 {
 	if (!gr_lock ()) {
 		fprintf (stderr, _("%s: unable to lock group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	if (!gr_open (O_RDWR)) {
 		fprintf (stderr, _("%s: unable to open group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
 	if (is_shadow_grp && !sgr_lock ()) {
 		fprintf (stderr,
 			 _("%s: unable to lock shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	if (is_shadow_grp && !sgr_open (O_RDWR)) {
 		fprintf (stderr,
 			 _("%s: unable to open shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 #endif				/* SHADOWGRP */
 }
@@ -188,7 +212,7 @@ static void group_busy (gid_t gid)
 	endpwent ();
 
 	/*
-	 * If pwd isn't NULL, it stopped becaues the gid's matched.
+	 * If pwd isn't NULL, it stopped because the gid's matched.
 	 */
 
 	if (pwd == (struct passwd *) 0)
@@ -198,7 +222,7 @@ static void group_busy (gid_t gid)
 	 * Can't remove the group.
 	 */
 	fprintf (stderr, _("%s: cannot remove user's primary group.\n"), Prog);
-	exit (E_GROUP_BUSY);
+	fail_exit (E_GROUP_BUSY);
 }
 
 /*
@@ -213,11 +237,8 @@ static void group_busy (gid_t gid)
 
 int main (int argc, char **argv)
 {
-	struct group *grp;
-
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -245,13 +266,17 @@ int main (int argc, char **argv)
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("groupdel", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("groupdel", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -278,20 +303,25 @@ int main (int argc, char **argv)
 	is_shadow_grp = sgr_file_present ();
 #endif
 
-	/*
-	 * Start with a quick check to see if the group exists.
-	 */
-	if (!(grp = getgrnam (group_name))) {
-		fprintf (stderr, _("%s: group %s does not exist\n"),
-			 Prog, group_name);
+	{
+		struct group *grp;
+		/*
+		 * Start with a quick check to see if the group exists.
+		 */
+		/* local, no need for xgetgrnam */
+		if (!(grp = getgrnam (group_name))) {
+			fprintf (stderr, _("%s: group %s does not exist\n"),
+				 Prog, group_name);
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
-			      group_name, -1, 0);
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "deleting group",
+				      group_name, -1, 0);
 #endif
-		exit (E_NOTFOUND);
-	}
+			exit (E_NOTFOUND);
+		}
 
-	group_id = grp->gr_gid;	/* LAUS */
+		group_id = grp->gr_gid;	/* LAUS */
+	}
 
 #ifdef	USE_NIS
 	/*
@@ -321,7 +351,7 @@ int main (int argc, char **argv)
 	 * Now check to insure that this isn't the primary group of
 	 * anyone.
 	 */
-	group_busy (grp->gr_gid);
+	group_busy (group_id);
 
 	/*
 	 * Do the hard stuff - open the files, delete the group entries,
@@ -330,6 +360,7 @@ int main (int argc, char **argv)
 	open_files ();
 
 	grp_update ();
+
 	close_files ();
 
 	nscd_flush_cache ("group");
@@ -338,12 +369,7 @@ int main (int argc, char **argv)
 	if (retval == PAM_SUCCESS)
 		pam_end (pamh, PAM_SUCCESS);
 #endif				/* USE_PAM */
-	if (errors != 0)
-#ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
-			      group_name, -1, 0);
-#endif
-	exit (errors == 0 ? E_SUCCESS : E_GRP_UPDATE);
-	/* NOT REACHED */
-	return 0;
+
+	return E_SUCCESS;
 }
+
