@@ -59,8 +59,6 @@
 /*
  * Global variables
  */
-extern int optind;
-extern char *optarg;
 static char *adduser = NULL;
 static char *deluser = NULL;
 static char *thisgroup = NULL;
@@ -76,14 +74,16 @@ static int isroot (void)
 static int isgroup (void)
 {
 	gid_t g = getgid ();
-	struct group *grp = getgrgid (g);
+	struct group *grp = getgrgid (g); /* local, no need for xgetgrgid */
 
 	return TRUE;
 }
 
 static char *whoami (void)
 {
+	/* local, no need for xgetgrgid */
 	struct group *grp = getgrgid (getgid ());
+	/* local, no need for xgetpwuid */
 	struct passwd *usr = getpwuid (getuid ());
 
 	if (0 == strcmp (usr->pw_name, grp->gr_name)) {
@@ -99,12 +99,12 @@ static void addtogroup (char *user, char **members)
 
 	for (i = 0; NULL != members[i]; i++) {
 		if (0 == strcmp (user, members[i])) {
-			fprintf (stderr, _("Member already exists\n"));
+			fputs (_("Member already exists\n"), stderr);
 			exit (EXIT_MEMBER_EXISTS);
 		}
 	}
 
-	members = (char **) realloc (members, sizeof (char *) * i);
+	members = (char **) realloc (members, sizeof (char *) * (i+2));
 	members[i] = user;
 	members[i + 1] = NULL;
 }
@@ -124,11 +124,12 @@ static void rmfromgroup (char *user, char **members)
 	}
 
 	while (found && NULL != members[i]) {
-		members[i] = members[++i];
+		members[i] = members[i+1];
+		i++;
 	}
 
 	if (!found) {
-		fprintf (stderr, _("Member to remove could not be found\n"));
+		fputs (_("Member to remove could not be found\n"), stderr);
 		exit (EXIT_NOT_MEMBER);
 	}
 }
@@ -159,9 +160,7 @@ static void members (char **members)
 
 static void usage (void)
 {
-	fprintf (stderr,
-		 _
-		 ("Usage: groupmems -a username | -d username | -D | -l [-g groupname]\n"));
+	fputs (_("Usage: groupmems -a username | -d username | -D | -l [-g groupname]\n"), stderr);
 	exit (EXIT_USAGE);
 }
 
@@ -173,7 +172,6 @@ int main (int argc, char **argv)
 
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -224,28 +222,32 @@ int main (int argc, char **argv)
 	}
 
 	if (!isroot () && NULL != thisgroup) {
-		fprintf (stderr,
-			 _("Only root can add members to different groups\n"));
+		fputs (_("Only root can add members to different groups\n"),
+		       stderr);
 		exit (EXIT_NOT_ROOT);
 	} else if (isroot () && NULL != thisgroup) {
 		name = thisgroup;
 	} else if (!isgroup ()) {
-		fprintf (stderr, _("Group access is required\n"));
+		fputs (_("Group access is required\n"), stderr);
 		exit (EXIT_NOT_EROOT);
 	} else if (NULL == (name = whoami ())) {
-		fprintf (stderr, _("Not primary owner of current group\n"));
+		fputs (_("Not primary owner of current group\n"), stderr);
 		exit (EXIT_NOT_PRIMARY);
 	}
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("groupmod", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("groupmod", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -263,18 +265,18 @@ int main (int argc, char **argv)
 	}
 
 	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("PAM authentication failed for\n"));
+		fputs (_("PAM authentication failed for\n"), stderr);
 		exit (1);
 	}
 #endif
 
 	if (!gr_lock ()) {
-		fprintf (stderr, _("Unable to lock group file\n"));
+		fputs (_("Unable to lock group file\n"), stderr);
 		exit (EXIT_GROUP_FILE);
 	}
 
 	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("Unable to open group file\n"));
+		fputs (_("Unable to open group file\n"), stderr);
 		exit (EXIT_GROUP_FILE);
 	}
 
@@ -294,7 +296,7 @@ int main (int argc, char **argv)
 	}
 
 	if (!gr_close ()) {
-		fprintf (stderr, _("Cannot close group file\n"));
+		fputs (_("Cannot close group file\n"), stderr);
 		exit (EXIT_GROUP_FILE);
 	}
 
