@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: groupmod.c,v 1.42 2006/08/15 17:25:58 kloczek Exp $"
+#ident "$Id: groupmod.c 1901 2008-03-17 23:02:23Z nekral-guest $"
 
 #include <ctype.h>
 #include <fcntl.h>
@@ -44,6 +44,7 @@
 #include "chkname.h"
 #include "defines.h"
 #include "groupio.h"
+#include "pwio.h"
 #include "nscd.h"
 #include "prototypes.h"
 #ifdef	SHADOWGRP
@@ -64,21 +65,27 @@
  */
 #ifdef	SHADOWGRP
 static int is_shadow_grp;
-#endif
+static int gshadow_locked = 0;
+#endif				/* SHADOWGRP */
+static int group_locked = 0;
+static int passwd_locked = 0;
 static char *group_name;
 static char *group_newname;
+static char *group_passwd;
 static gid_t group_id;
 static gid_t group_newid;
 
 static char *Prog;
 
 static int
- oflg = 0,			/* permit non-unique group ID to be specified with -g */
+    oflg = 0,			/* permit non-unique group ID to be specified with -g */
     gflg = 0,			/* new ID value for the group */
-    nflg = 0;			/* a new name has been specified for the group */
+    nflg = 0,			/* a new name has been specified for the group */
+    pflg = 0;			/* new encrypted password */
 
 /* local function prototypes */
 static void usage (void);
+static void fail_exit (int);
 static void new_grent (struct group *);
 
 #ifdef SHADOWGRP
@@ -91,6 +98,7 @@ static void process_flags (int, char **);
 static void close_files (void);
 static void open_files (void);
 static gid_t get_gid (const char *gidstr);
+static void update_primary_groups (gid_t ogid, gid_t ngid);
 
 /*
  * usage - display usage message and exit
@@ -98,17 +106,34 @@ static gid_t get_gid (const char *gidstr);
 
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: groupmod [options] GROUP\n"
-			   "\n"
-			   "Options:\n"
-			   "  -g, --gid GID			force use new GID by GROUP\n"
-			   "  -h, --help			display this help message and exit\n"
-			   "  -n, --new-name NEW_GROUP	force use NEW_GROUP name by GROUP\n"
-			   "  -o, --non-unique		allow using duplicate (non-unique) GID by GROUP\n"
-			   "\n"));
+	fputs (_("Usage: groupmod [options] GROUP\n"
+	         "\n"
+	         "Options:\n"
+	         "  -g, --gid GID                 force use new GID by GROUP\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -n, --new-name NEW_GROUP      force use NEW_GROUP name by GROUP\n"
+	         "  -o, --non-unique              allow using duplicate (non-unique) GID by GROUP\n"
+	         "  -p, --password PASSWORD       use encrypted password for the new password\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
+static void fail_exit (int status)
+{
+	if (group_locked) {
+		gr_unlock ();
+	}
+#ifdef	SHADOWGRP
+	if (gshadow_locked) {
+		sgr_unlock ();
+	}
+#endif				/* SHADOWGRP */
+	if (passwd_locked) {
+		pw_unlock();
+	}
+	exit (status);
+}
+
 /*
  * new_grent - updates the values in a group file entry
  *
@@ -122,6 +147,9 @@ static void new_grent (struct group *grent)
 
 	if (gflg)
 		grent->gr_gid = group_newid;
+
+	if (pflg)
+		grent->gr_passwd = group_passwd;
 }
 
 #ifdef	SHADOWGRP
@@ -135,6 +163,9 @@ static void new_sgent (struct sgrp *sgent)
 {
 	if (nflg)
 		sgent->sg_name = xstrdup (group_newname);
+
+	if (pflg)
+		sgent->sg_passwd = group_passwd;
 }
 #endif				/* SHADOWGRP */
 
@@ -165,7 +196,7 @@ static void grp_update (void)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	grp = *ogrp;
 	new_grent (&grp);
@@ -173,9 +204,15 @@ static void grp_update (void)
 	if (is_shadow_grp && (osgrp = sgr_locate (group_name))) {
 		sgrp = *osgrp;
 		new_sgent (&sgrp);
+		if (pflg)
+			grp.gr_passwd = SHADOW_PASSWD_STRING;
 	}
 #endif				/* SHADOWGRP */
 
+	if (gflg) {
+		update_primary_groups (ogrp->gr_gid, group_newid);
+	}
+
 	/*
 	 * Write out the new group file entry.
 	 */
@@ -185,7 +222,7 @@ static void grp_update (void)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	if (nflg && !gr_remove (group_name)) {
 		fprintf (stderr, _("%s: error removing group entry\n"), Prog);
@@ -193,7 +230,7 @@ static void grp_update (void)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
 
@@ -208,21 +245,21 @@ static void grp_update (void)
 	/*
 	 * Write out the new shadow group entries as well.
 	 */
-	if (!sgr_update (&sgrp)) {
+	if (is_shadow_grp && !sgr_update (&sgrp)) {
 		fprintf (stderr, _("%s: error adding new group entry\n"), Prog);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
-	if (nflg && !sgr_remove (group_name)) {
+	if (is_shadow_grp && nflg && !sgr_remove (group_name)) {
 		fprintf (stderr, _("%s: error removing group entry\n"), Prog);
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "deleting group",
 			      group_name, -1, 0);
 #endif
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
       out:
 #endif				/* SHADOWGRP */
@@ -235,9 +272,10 @@ static void grp_update (void)
 		SYSLOG ((LOG_INFO, "change group `%s' to `%s'",
 			 group_name, group_newname));
 
-	if (gflg)
+	if (gflg) {
 		SYSLOG ((LOG_INFO, "change GID for `%s' to %u",
 			 nflg ? group_newname : group_name, group_newid));
+	}
 }
 
 /*
@@ -257,7 +295,7 @@ static void check_new_gid (void)
 		return;
 	}
 
-	if (oflg || !getgrgid (group_newid))
+	if (oflg || !getgrgid (group_newid)) /* local, no need for xgetgrgid */
 		return;
 
 	/*
@@ -268,7 +306,7 @@ static void check_new_gid (void)
 	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modify gid", NULL,
 		      group_newid, 0);
 #endif
-	exit (E_GID_IN_USE);
+	fail_exit (E_GID_IN_USE);
 }
 
 /*
@@ -292,6 +330,7 @@ static void check_new_name (void)
 		/*
 		 * If the entry is found, too bad.
 		 */
+		/* local, no need for xgetgrnam */
 		if (getgrnam (group_newname)) {
 			fprintf (stderr,
 				 _("%s: %s is not a unique name\n"), Prog,
@@ -300,7 +339,7 @@ static void check_new_name (void)
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 				      "modifying group", group_name, -1, 0);
 #endif
-			exit (E_NAME_IN_USE);
+			fail_exit (E_NAME_IN_USE);
 		}
 		return;
 	}
@@ -315,7 +354,7 @@ static void check_new_name (void)
 	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group", group_name,
 		      -1, 0);
 #endif
-	exit (E_BAD_ARG);
+	fail_exit (E_BAD_ARG);
 }
 
 /*
@@ -330,7 +369,7 @@ static gid_t get_gid (const char *gidstr)
 	if (*errptr || errno == ERANGE || val < 0) {
 		fprintf (stderr, _("%s: invalid numeric argument '%s'\n"), Prog,
 			 gidstr);
-		exit (E_BAD_ARG);
+		fail_exit (E_BAD_ARG);
 	}
 	return val;
 }
@@ -353,10 +392,11 @@ static void process_flags (int argc, char **argv)
 			{"help", no_argument, NULL, 'h'},
 			{"new-name", required_argument, NULL, 'n'},
 			{"non-unique", no_argument, NULL, 'o'},
+			{"password", required_argument, NULL, 'p'},
 			{NULL, 0, NULL, '\0'}
 		};
 		while ((c =
-			getopt_long (argc, argv, "g:hn:o",
+			getopt_long (argc, argv, "g:hn:op:",
 				     long_options, &option_index)) != -1) {
 			switch (c) {
 			case 'g':
@@ -375,6 +415,10 @@ static void process_flags (int argc, char **argv)
 			case 'o':
 				oflg++;
 				break;
+			case 'p':
+				group_passwd = optarg;
+				pflg++;
+				break;
 			default:
 				usage ();
 			}
@@ -400,18 +444,30 @@ static void close_files (void)
 {
 	if (!gr_close ()) {
 		fprintf (stderr, _("%s: cannot rewrite group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 	gr_unlock ();
+	group_locked--;
 #ifdef	SHADOWGRP
 	if (is_shadow_grp && !sgr_close ()) {
 		fprintf (stderr,
 			 _("%s: cannot rewrite shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
-	if (is_shadow_grp)
+	if (is_shadow_grp) {
 		sgr_unlock ();
+		gshadow_locked--;
+	}
 #endif				/* SHADOWGRP */
+	if (gflg) {
+		if (!pw_close ()) {
+			fprintf (stderr,
+			         _("%s: cannot rewrite passwd file\n"), Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		pw_unlock();
+		passwd_locked--;
+	}
 }
 
 /*
@@ -423,24 +479,74 @@ static void open_files (void)
 {
 	if (!gr_lock ()) {
 		fprintf (stderr, _("%s: unable to lock group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
+	group_locked++;
 	if (!gr_open (O_RDWR)) {
 		fprintf (stderr, _("%s: unable to open group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
-	if (is_shadow_grp && !sgr_lock ()) {
-		fprintf (stderr,
-			 _("%s: unable to lock shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
-	}
-	if (is_shadow_grp && !sgr_open (O_RDWR)) {
-		fprintf (stderr,
-			 _("%s: unable to open shadow group file\n"), Prog);
-		exit (E_GRP_UPDATE);
+	if (is_shadow_grp) {
+		if (!sgr_lock ()) {
+			fprintf (stderr,
+			         _("%s: unable to lock shadow group file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		gshadow_locked++;
+		if (!sgr_open (O_RDWR)) {
+			fprintf (stderr,
+			         _("%s: unable to open shadow group file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
 	}
 #endif				/* SHADOWGRP */
+	if (gflg) {
+		if (!pw_lock ()) {
+			fprintf (stderr,
+			         _("%s: unable to lock password file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		passwd_locked++;
+		if (!pw_open (O_RDWR)) {
+			fprintf (stderr,
+			         _("%s: unable to open password file\n"),
+			         Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+	}
+}
+
+void update_primary_groups (gid_t ogid, gid_t ngid)
+{
+	struct passwd *pwd;
+
+	setpwent ();
+	while ((pwd = getpwent ()) != NULL) {
+		if (pwd->pw_gid == ogid) {
+			const struct passwd *lpwd;
+			struct passwd npwd;
+			lpwd = pw_locate (pwd->pw_name);
+			if (NULL == lpwd) {
+				fprintf (stderr,
+				         _("%s: cannot change the primary group of user '%s' from %u to %u, since it is not in the passwd file.\n"),
+				         Prog, pwd->pw_name, ogid, ngid);
+				fail_exit (E_GRP_UPDATE);
+			} else {
+				npwd = *lpwd;
+				npwd.pw_gid = ngid;
+				if (!pw_update (&npwd)) {
+					fprintf (stderr,
+					         _("%s: cannot change the primary group of user '%s' from %u to %u.\n"),
+					         Prog, pwd->pw_name, ogid, ngid);
+					fail_exit (E_GRP_UPDATE);
+				}
+			}
+		}
+	}
 }
 
 /*
@@ -457,11 +563,8 @@ static void open_files (void)
  */
 int main (int argc, char **argv)
 {
-	struct group *grp;
-
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
-	struct passwd *pampw;
 	int retval;
 #endif
 
@@ -485,13 +588,17 @@ int main (int argc, char **argv)
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid (getuid ());
-	if (pampw == NULL) {
-		retval = PAM_USER_UNKNOWN;
-	}
+	{
+		struct passwd *pampw;
+		pampw = getpwuid (getuid ()); /* local, no need for xgetpwuid */
+		if (pampw == NULL) {
+			retval = PAM_USER_UNKNOWN;
+		}
 
-	if (retval == PAM_SUCCESS) {
-		retval = pam_start ("groupmod", pampw->pw_name, &conv, &pamh);
+		if (retval == PAM_SUCCESS) {
+			retval = pam_start ("groupmod", pampw->pw_name,
+					    &conv, &pamh);
+		}
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -510,26 +617,30 @@ int main (int argc, char **argv)
 
 	if (retval != PAM_SUCCESS) {
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
-		exit (1);
+		fail_exit (1);
 	}
 #endif				/* USE_PAM */
 
 #ifdef SHADOWGRP
 	is_shadow_grp = sgr_file_present ();
 #endif
-	/*
-	 * Start with a quick check to see if the group exists.
-	 */
-	if (!(grp = getgrnam (group_name))) {
-		fprintf (stderr, _("%s: group %s does not exist\n"),
-			 Prog, group_name);
+	{
+		struct group *grp;
+		/*
+		 * Start with a quick check to see if the group exists.
+		 */
+		/* local, no need for xgetgrnam */
+		if (!(grp = getgrnam (group_name))) {
+			fprintf (stderr, _("%s: group %s does not exist\n"),
+				 Prog, group_name);
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "modifying group",
-			      group_name, -1, 0);
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "modifying group", group_name, -1, 0);
 #endif
-		exit (E_NOTFOUND);
-	} else
-		group_id = grp->gr_gid;
+			fail_exit (E_NOTFOUND);
+		} else
+			group_id = grp->gr_gid;
+	}
 
 #ifdef WITH_AUDIT
 	/* Set new name/id to original if not specified on command line */
@@ -559,7 +670,7 @@ int main (int argc, char **argv)
 			fprintf (stderr, _("%s: %s is the NIS master\n"),
 				 Prog, nis_master);
 		}
-		exit (E_NOTFOUND);
+		fail_exit (E_NOTFOUND);
 	}
 #endif
 
@@ -576,6 +687,7 @@ int main (int argc, char **argv)
 	open_files ();
 
 	grp_update ();
+
 	close_files ();
 
 	nscd_flush_cache ("group");
@@ -587,3 +699,4 @@ int main (int argc, char **argv)
 	exit (E_SUCCESS);
 	/* NOT REACHED */
 }
+
