@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: grpck.c,v 1.32 2006/05/07 17:44:39 kloczek Exp $"
+#ident "$Id: grpck.c 1900 2008-03-17 23:00:49Z nekral-guest $"
 
 #include <fcntl.h>
 #include <grp.h>
@@ -41,13 +41,9 @@
 #include "groupio.h"
 #include "nscd.h"
 #include "prototypes.h"
-extern void __gr_del_entry (const struct commonio_entry *);
-extern struct commonio_entry *__gr_get_head (void);
 
 #ifdef SHADOWGRP
 #include "sgroupio.h"
-extern void __sgr_del_entry (const struct commonio_entry *);
-extern struct commonio_entry *__sgr_get_head (void);
 #endif
 
 /*
@@ -66,16 +62,38 @@ extern struct commonio_entry *__sgr_get_head (void);
  */
 static char *Prog;
 static const char *grp_file = GROUP_FILE;
+static int use_system_grp_file = 1;
 
 #ifdef	SHADOWGRP
 static const char *sgr_file = SGROUP_FILE;
+static int use_system_sgr_file = 1;
+static int is_shadow = 0;
 #endif
+/* Options */
 static int read_only = 0;
+static int sort_mode = 0;
 
 /* local function prototypes */
 static void usage (void);
-static int yes_or_no (void);
 static void delete_member (char **, const char *);
+static void process_flags (int argc, char **argv);
+static void open_files (void);
+static void close_files (int changed);
+static int check_members (const char *groupname,
+                          char **members,
+                          const char *fmt_info,
+                          const char *fmt_prompt,
+                          const char *fmt_syslog,
+                          int *errors);
+static void check_grp_file (int *errors, int *changed);
+#ifdef SHADOWGRP
+static void compare_members_lists (const char *groupname,
+                                   char **members,
+                                   char **other_members,
+                                   const char *file,
+                                   const char *other_file);
+static void check_sgr_file (int *errors, int *changed);
+#endif
 
 /*
  * usage - print syntax message and exit
@@ -90,30 +108,6 @@ static void usage (void)
 	exit (E_USAGE);
 }
 
-/*
- * yes_or_no - get answer to question from the user
- */
-static int yes_or_no (void)
-{
-	char buf[80];
-
-	/*
-	 * In read-only mode all questions are answered "no".
-	 */
-	if (read_only) {
-		printf (_("No\n"));
-		return 0;
-	}
-
-	/*
-	 * Get a line and see what the first character is.
-	 */
-	if (fgets (buf, sizeof buf, stdin))
-		return buf[0] == 'y' || buf[0] == 'Y';
-
-	return 0;
-}
-
 /*
  * delete_member - delete an entry in a list of members
  */
@@ -121,44 +115,27 @@ static void delete_member (char **list, const char *member)
 {
 	int i;
 
-	for (i = 0; list[i]; i++)
-		if (list[i] == member)
+	for (i = 0; list[i]; i++) {
+		if (list[i] == member) {
 			break;
+		}
+	}
 
-	if (list[i])
-		for (; list[i]; i++)
+	if (list[i]) {
+		for (; list[i]; i++) {
 			list[i] = list[i + 1];
+		}
+	}
 }
 
 /*
- * grpck - verify group file integrity
+ * process_flags - parse the command line options
+ *
+ *	It will not return if an error is encountered.
  */
-int main (int argc, char **argv)
+static void process_flags (int argc, char **argv)
 {
 	int arg;
-	int errors = 0;
-	int changed = 0;
-	int i;
-	struct commonio_entry *gre, *tgre;
-	struct group *grp;
-	int sort_mode = 0;
-
-#ifdef	SHADOWGRP
-	struct commonio_entry *sge, *tsge;
-	struct sgrp *sgr;
-	int is_shadow = 0;
-#endif
-
-	/*
-	 * Get my name so that I can use it to report errors.
-	 */
-	Prog = Basename (argv[0]);
-
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
-
-	OPENLOG ("grpck");
 
 	/*
 	 * Parse the command line arguments
@@ -188,11 +165,13 @@ int main (int argc, char **argv)
 	 * Make certain we have the right number of arguments
 	 */
 #ifdef	SHADOWGRP
-	if (optind != argc && optind + 1 != argc && optind + 2 != argc)
+	if ((argc < optind) || (argc > (optind + 2)))
 #else
-	if (optind != argc && optind + 1 != argc)
+	if ((argc < optind) || (argc > (optind + 1)))
 #endif
+	{
 		usage ();
+	}
 
 	/*
 	 * If there are two left over filenames, use those as the group and
@@ -201,34 +180,48 @@ int main (int argc, char **argv)
 	if (optind != argc) {
 		grp_file = argv[optind];
 		gr_name (grp_file);
+		use_system_grp_file = 0;
 	}
 #ifdef	SHADOWGRP
-	if (optind + 2 == argc) {
+	if ((optind + 2) == argc) {
 		sgr_file = argv[optind + 1];
 		sgr_name (sgr_file);
 		is_shadow = 1;
-	} else if (optind == argc)
+		use_system_sgr_file = 0;
+	} else if (optind == argc) {
 		is_shadow = sgr_file_present ();
+	}
 #endif
+}
 
+/*
+ * open_files - open the shadow database
+ *
+ *	In read-only mode, the databases are not locked and are opened
+ *	only for reading.
+ */
+static void open_files (void)
+{
 	/*
 	 * Lock the files if we aren't in "read-only" mode
 	 */
 	if (!read_only) {
-		if (!gr_lock ()) {
+		if (gr_lock () == 0) {
 			fprintf (stderr, _("%s: cannot lock file %s\n"),
-				 Prog, grp_file);
-			if (optind == argc)
+			         Prog, grp_file);
+			if (use_system_grp_file) {
 				SYSLOG ((LOG_WARN, "cannot lock %s", grp_file));
+			}
 			closelog ();
 			exit (E_CANT_LOCK);
 		}
 #ifdef	SHADOWGRP
-		if (is_shadow && !sgr_lock ()) {
+		if (is_shadow && (sgr_lock () == 0)) {
 			fprintf (stderr, _("%s: cannot lock file %s\n"),
-				 Prog, sgr_file);
-			if (optind == argc)
+			         Prog, sgr_file);
+			if (use_system_sgr_file) {
 				SYSLOG ((LOG_WARN, "cannot lock %s", sgr_file));
+			}
 			closelog ();
 			exit (E_CANT_LOCK);
 		}
@@ -239,65 +232,202 @@ int main (int argc, char **argv)
 	 * Open the files. Use O_RDONLY if we are in read_only mode,
 	 * O_RDWR otherwise.
 	 */
-	if (!gr_open (read_only ? O_RDONLY : O_RDWR)) {
+	if (gr_open (read_only ? O_RDONLY : O_RDWR) == 0) {
 		fprintf (stderr, _("%s: cannot open file %s\n"), Prog,
-			 grp_file);
-		if (optind == argc)
+		         grp_file);
+		if (use_system_grp_file) {
 			SYSLOG ((LOG_WARN, "cannot open %s", grp_file));
+		}
 		closelog ();
 		exit (E_CANT_OPEN);
 	}
 #ifdef	SHADOWGRP
-	if (is_shadow && !sgr_open (read_only ? O_RDONLY : O_RDWR)) {
+	if (is_shadow && (sgr_open (read_only ? O_RDONLY : O_RDWR) == 0)) {
 		fprintf (stderr, _("%s: cannot open file %s\n"), Prog,
-			 sgr_file);
-		if (optind == argc)
+		         sgr_file);
+		if (use_system_sgr_file) {
 			SYSLOG ((LOG_WARN, "cannot open %s", sgr_file));
+		}
 		closelog ();
 		exit (E_CANT_OPEN);
 	}
 #endif
+}
 
-	if (sort_mode) {
-		gr_sort ();
+/*
+ * close_files - close and unlock the group/gshadow databases
+ *
+ *	If changed is not set, the databases are not closed, and no
+ *	changes are committed in the databases. The databases are
+ *	unlocked anyway.
+ */
+static void close_files (int changed)
+{
+	/*
+	 * All done. If there were no change we can just abandon any
+	 * changes to the files.
+	 */
+	if (changed) {
+		if (gr_close () == 0) {
+			fprintf (stderr, _("%s: cannot update file %s\n"),
+			         Prog, grp_file);
+			exit (E_CANT_UPDATE);
+		}
 #ifdef	SHADOWGRP
-		if (is_shadow)
-			sgr_sort ();
+		if (is_shadow && (sgr_close () == 0)) {
+			fprintf (stderr, _("%s: cannot update file %s\n"),
+			         Prog, sgr_file);
+			exit (E_CANT_UPDATE);
+		}
 #endif
-		goto write_and_bye;
 	}
 
+	/*
+	 * Don't be anti-social - unlock the files when you're done.
+	 */
+#ifdef	SHADOWGRP
+	if (is_shadow) {
+		sgr_unlock ();
+	}
+#endif
+	(void) gr_unlock ();
+}
+
+/*
+ * check_members - check that every members of a group exist
+ *
+ *	If an error is detected, *errors is incremented.
+ *
+ *	The user will be prompted for the removal of the non-existent
+ *	user.
+ *
+ *	If any changes are performed, the return value will be 1,
+ *	otherwise check_members() returns 0.
+ *
+ *	fmt_info, fmt_prompt, and fmt_syslog are used for logging.
+ *	  * fmt_info must contain two string flags (%s): for the group's
+ *	    name and the missing member.
+ *	  * fmt_prompt must contain one string flags (%s): the missing
+ *	    member.
+ *	  * fmt_syslog must contain two string flags (%s): for the
+ *	    group's name and the missing member.
+ */
+static int check_members (const char *groupname,
+                          char **members,
+                          const char *fmt_info,
+                          const char *fmt_prompt,
+                          const char *fmt_syslog,
+                          int *errors)
+{
+	int i;
+	int members_changed = 0;
+
+	/*
+	 * Make sure each member exists
+	 */
+	for (i = 0; NULL != members[i]; i++) {
+		/* local, no need for xgetpwnam */
+		if (getpwnam (members[i]) != NULL) {
+			continue;
+		}
+		/*
+		 * Can't find this user. Remove them
+		 * from the list.
+		 */
+		*errors += 1;
+		printf (fmt_info, groupname, members[i]);
+		printf (fmt_prompt, members[i]);
+
+		if (!yes_or_no (read_only)) {
+			continue;
+		}
+
+		SYSLOG ((LOG_INFO, fmt_syslog, members[i], groupname));
+		members_changed = 1;
+		delete_member (members, members[i]);
+	}
+
+	return members_changed;
+}
+
+#ifdef SHADOWGRP
+/*
+ * compare_members_lists - make sure the list of members is contained in
+ *                         another list.
+ *
+ *	compare_members_lists() checks that all the members of members are
+ *	also in other_members.
+ *	file and other_file are used for logging.
+ *
+ *	TODO: No changes are performed on the lists.
+ */
+static void compare_members_lists (const char *groupname,
+                                   char **members,
+                                   char **other_members,
+                                   const char *file,
+                                   const char *other_file)
+{
+	char **pmem, **other_pmem;
+
+	for (pmem = members; NULL != *pmem; pmem++) {
+		for (other_pmem = other_members; NULL != *other_pmem; other_pmem++) {
+			if (strcmp (*pmem, *other_pmem) == 0) {
+				break;
+			}
+		}
+		if (*other_pmem == NULL) {
+			printf
+			    ("'%s' is a member of the '%s' group in %s but not in %s\n",
+			     *pmem, groupname, file, other_file);
+		}
+	}
+}
+#endif				/* SHADOWGRP */
+
+/*
+ * check_grp_file - check the content of the group file
+ */
+static void check_grp_file (int *errors, int *changed)
+{
+	struct commonio_entry *gre, *tgre;
+	struct group *grp;
+#ifdef SHADOWGRP
+	struct sgrp *sgr;
+#endif
+
 	/*
 	 * Loop through the entire group file.
 	 */
-	for (gre = __gr_get_head (); gre; gre = gre->next) {
+	for (gre = __gr_get_head (); NULL != gre; gre = gre->next) {
 		/*
 		 * Skip all NIS entries.
 		 */
 
-		if (gre->line[0] == '+' || gre->line[0] == '-')
+		if ((gre->line[0] == '+') || (gre->line[0] == '-')) {
 			continue;
+		}
 
 		/*
 		 * Start with the entries that are completely corrupt. They
 		 * have no (struct group) entry because they couldn't be
 		 * parsed properly.
 		 */
-		if (!gre->eptr) {
+		if (NULL == gre->eptr) {
 
 			/*
 			 * Tell the user this entire line is bogus and ask
 			 * them to delete it.
 			 */
-			printf (_("invalid group file entry\n"));
+			puts (_("invalid group file entry"));
 			printf (_("delete line '%s'? "), gre->line);
-			errors++;
+			*errors += 1;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-			if (!yes_or_no ())
+			if (!yes_or_no (read_only)) {
 				continue;
+			}
 
 			/*
 			 * All group file deletions wind up here. This code
@@ -307,8 +437,8 @@ int main (int argc, char **argv)
 			 */
 		      delete_gr:
 			SYSLOG ((LOG_INFO, "delete group line `%s'",
-				 gre->line));
-			changed++;
+			         gre->line));
+			*changed = 1;
 
 			__gr_del_entry (gre);
 			continue;
@@ -322,45 +452,49 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure this entry has a unique name.
 		 */
-		for (tgre = __gr_get_head (); tgre; tgre = tgre->next) {
+		for (tgre = __gr_get_head (); NULL != tgre; tgre = tgre->next) {
 
 			const struct group *ent = tgre->eptr;
 
 			/*
 			 * Don't check this entry
 			 */
-			if (tgre == gre)
+			if (tgre == gre) {
 				continue;
+			}
 
 			/*
 			 * Don't check invalid entries.
 			 */
-			if (!ent)
+			if (NULL == ent) {
 				continue;
+			}
 
-			if (strcmp (grp->gr_name, ent->gr_name) != 0)
+			if (strcmp (grp->gr_name, ent->gr_name) != 0) {
 				continue;
+			}
 
 			/*
 			 * Tell the user this entry is a duplicate of
 			 * another and ask them to delete it.
 			 */
-			printf (_("duplicate group entry\n"));
+			puts (_("duplicate group entry"));
 			printf (_("delete line '%s'? "), gre->line);
-			errors++;
+			*errors += 1;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-			if (yes_or_no ())
+			if (yes_or_no (read_only)) {
 				goto delete_gr;
+			}
 		}
 
 		/*
 		 * Check for invalid group names.  --marekm
 		 */
 		if (!check_group_name (grp->gr_name)) {
-			errors++;
+			*errors += 1;
 			printf (_("invalid group name '%s'\n"), grp->gr_name);
 		}
 
@@ -369,32 +503,18 @@ int main (int argc, char **argv)
 		 * groups with no members are returned as groups with one
 		 * member "", causing grpck to fail.  --marekm
 		 */
-		if (grp->gr_mem[0] && !grp->gr_mem[1]
-		    && *(grp->gr_mem[0]) == '\0')
-			grp->gr_mem[0] = (char *) 0;
+		if (   (NULL != grp->gr_mem[0])
+		    && (NULL == grp->gr_mem[1])
+		    && ('\0' == grp->gr_mem[0][0])) {
+			grp->gr_mem[0] = NULL;
+		}
 
-		/*
-		 * Make sure each member exists
-		 */
-		for (i = 0; grp->gr_mem[i]; i++) {
-			if (getpwnam (grp->gr_mem[i]))
-				continue;
-			/*
-			 * Can't find this user. Remove them
-			 * from the list.
-			 */
-			errors++;
-			printf (_("group %s: no user %s\n"),
-				grp->gr_name, grp->gr_mem[i]);
-			printf (_("delete member '%s'? "), grp->gr_mem[i]);
-
-			if (!yes_or_no ())
-				continue;
-
-			SYSLOG ((LOG_INFO, "delete member '%s' group '%s'",
-				 grp->gr_mem[i], grp->gr_name));
-			changed++;
-			delete_member (grp->gr_mem, grp->gr_mem[i]);
+		if (check_members (grp->gr_name, grp->gr_mem,
+		                   _("group %s: no user %s\n"),
+		                   _("delete member '%s'? "),
+		                   "delete member `%s' from group `%s'",
+		                   errors) == 1) {
+			*changed = 1;
 			gre->changed = 1;
 			__gr_set_changed ();
 		}
@@ -408,12 +528,12 @@ int main (int argc, char **argv)
 			sgr = (struct sgrp *) sgr_locate (grp->gr_name);
 			if (sgr == NULL) {
 				printf (_
-					("no matching group file entry in %s\n"),
-					sgr_file);
+				        ("no matching group file entry in %s\n"),
+				        sgr_file);
 				printf (_("add group '%s' in %s ?"),
-					grp->gr_name, sgr_file);
-				errors++;
-				if (yes_or_no ()) {
+				        grp->gr_name, sgr_file);
+				*errors += 1;
+				if (yes_or_no (read_only)) {
 					struct sgrp sg;
 					struct group gr;
 					static char *empty = NULL;
@@ -423,85 +543,79 @@ int main (int argc, char **argv)
 					sg.sg_adm = &empty;
 					sg.sg_mem = grp->gr_mem;
 					SYSLOG ((LOG_INFO,
-						 "add group `%s' to `%s'",
-						 grp->gr_name, sgr_file));
-					changed++;
+					         "add group `%s' to `%s'",
+					         grp->gr_name, sgr_file));
+					*changed = 1;
 
-					if (!sgr_update (&sg)) {
+					if (sgr_update (&sg) == 0) {
 						fprintf (stderr,
-							 _
-							 ("%s: can't update shadow entry for %s\n"),
-							 Prog, sg.sg_name);
+						         _
+						         ("%s: can't update shadow entry for %s\n"),
+						         Prog, sg.sg_name);
 						exit (E_CANT_UPDATE);
 					}
 					/* remove password from /etc/group */
 					gr = *grp;
 					gr.gr_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
-					if (!gr_update (&gr)) {
+					if (gr_update (&gr) == 0) {
 						fprintf (stderr,
-							 _
-							 ("%s: can't update entry for group %s\n"),
-							 Prog, gr.gr_name);
+						         _
+						         ("%s: can't update entry for group %s\n"),
+						         Prog, gr.gr_name);
 						exit (E_CANT_UPDATE);
 					}
 				}
 			} else {
 				/**
-				 * Verify that the all members defined in /etc/group are also
+				 * Verify that all the members defined in /etc/group are also
 				 * present in /etc/gshadow.
 				 */
-				char **pgrp_mem, **psgr_mem;
-
-				for (pgrp_mem = grp->gr_mem; *pgrp_mem;
-				     pgrp_mem++) {
-					for (psgr_mem = sgr->sg_mem; *psgr_mem;
-					     psgr_mem++) {
-						if (strcmp
-						    (*pgrp_mem, *psgr_mem) == 0)
-							break;
-					}
-					if (*psgr_mem == NULL) {
-						printf
-						    ("'%s' is a member of the '%s' group in %s but not in %s\n",
-						     *pgrp_mem, sgr->sg_name,
-						     grp_file, sgr_file);
-					}
-				}
+				compare_members_lists (grp->gr_name,
+				                       grp->gr_mem, sgr->sg_mem,
+				                       grp_file, sgr_file);
 			}
 		}
 #endif
 
 	}
+}
 
-#ifdef	SHADOWGRP
-	if (!is_shadow)
-		goto shadow_done;
+#ifdef SHADOWGRP
+/*
+ * check_sgr_file - check the content of the shadowed group file (gshadow)
+ */
+static void check_sgr_file (int *errors, int *changed)
+{
+	struct group *grp;
+	struct commonio_entry *sge, *tsge;
+	struct sgrp *sgr;
 
 	/*
 	 * Loop through the entire shadow group file.
 	 */
-	for (sge = __sgr_get_head (); sge; sge = sge->next) {
+	for (sge = __sgr_get_head (); NULL != sge; sge = sge->next) {
 
 		/*
 		 * Start with the entries that are completely corrupt. They
 		 * have no (struct sgrp) entry because they couldn't be
 		 * parsed properly.
 		 */
-		if (!sge->eptr) {
+		if (NULL == sge->eptr) {
 
 			/*
 			 * Tell the user this entire line is bogus and ask
 			 * them to delete it.
 			 */
-			printf (_("invalid shadow group file entry\n"));
+			puts (_("invalid shadow group file entry"));
 			printf (_("delete line '%s'? "), sge->line);
-			errors++;
+			*errors += 1;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-			if (!yes_or_no ())
+			if (!yes_or_no (read_only)) {
 				continue;
+			}
 
 			/*
 			 * All shadow group file deletions wind up here. 
@@ -511,8 +625,8 @@ int main (int argc, char **argv)
 			 */
 		      delete_sg:
 			SYSLOG ((LOG_INFO, "delete shadow line `%s'",
-				 sge->line));
-			changed++;
+			         sge->line));
+			*changed = 1;
 
 			__sgr_del_entry (sge);
 			continue;
@@ -526,38 +640,42 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure this entry has a unique name.
 		 */
-		for (tsge = __sgr_get_head (); tsge; tsge = tsge->next) {
+		for (tsge = __sgr_get_head (); NULL != tsge; tsge = tsge->next) {
 
 			const struct sgrp *ent = tsge->eptr;
 
 			/*
 			 * Don't check this entry
 			 */
-			if (tsge == sge)
+			if (tsge == sge) {
 				continue;
+			}
 
 			/*
 			 * Don't check invalid entries.
 			 */
-			if (!ent)
+			if (NULL == ent) {
 				continue;
+			}
 
-			if (strcmp (sgr->sg_name, ent->sg_name) != 0)
+			if (strcmp (sgr->sg_name, ent->sg_name) != 0) {
 				continue;
+			}
 
 			/*
 			 * Tell the user this entry is a duplicate of
 			 * another and ask them to delete it.
 			 */
-			printf (_("duplicate shadow group entry\n"));
+			puts (_("duplicate shadow group entry"));
 			printf (_("delete line '%s'? "), sge->line);
-			errors++;
+			*errors += 1;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-			if (yes_or_no ())
+			if (yes_or_no (read_only)) {
 				goto delete_sg;
+			}
 		}
 
 		/*
@@ -566,58 +684,31 @@ int main (int argc, char **argv)
 		grp = (struct group *) gr_locate (sgr->sg_name);
 		if (grp == NULL) {
 			printf (_("no matching group file entry in %s\n"),
-				grp_file);
+			        grp_file);
 			printf (_("delete line '%s'? "), sge->line);
-			errors++;
-			if (yes_or_no ())
+			*errors += 1;
+			if (yes_or_no (read_only)) {
 				goto delete_sg;
+			}
 		} else {
 			/**
 			 * Verify that the all members defined in /etc/gshadow are also
 			 * present in /etc/group.
 			 */
-			char **pgrp_mem, **psgr_mem;
-
-			for (psgr_mem = sgr->sg_mem; *psgr_mem; psgr_mem++) {
-				for (pgrp_mem = grp->gr_mem; *pgrp_mem;
-				     pgrp_mem++) {
-					if (strcmp (*pgrp_mem, *psgr_mem) == 0)
-						break;
-				}
-				if (*pgrp_mem == NULL) {
-					printf
-					    ("'%s' is a member of the '%s' group in %s but not in %s\n",
-					     *psgr_mem, sgr->sg_name, sgr_file,
-					     grp_file);
-				}
-			}
+			compare_members_lists (sgr->sg_name,
+			                       sgr->sg_mem, grp->gr_mem,
+			                       sgr_file, grp_file);
 		}
 
 		/*
 		 * Make sure each administrator exists
 		 */
-		for (i = 0; sgr->sg_adm[i]; i++) {
-			if (getpwnam (sgr->sg_adm[i]))
-				continue;
-			/*
-			 * Can't find this user. Remove them
-			 * from the list.
-			 */
-			errors++;
-			printf (_
-				("shadow group %s: no administrative user %s\n"),
-				sgr->sg_name, sgr->sg_adm[i]);
-			printf (_("delete administrative member '%s'? "),
-				sgr->sg_adm[i]);
-
-			if (!yes_or_no ())
-				continue;
-
-			SYSLOG ((LOG_INFO,
-				 "delete admin `%s' from shadow group `%s'",
-				 sgr->sg_adm[i], sgr->sg_name));
-			changed++;
-			delete_member (sgr->sg_adm, sgr->sg_adm[i]);
+		if (check_members (sgr->sg_name, sgr->sg_adm,
+		                   _("shadow group %s: no administrative user %s\n"),
+		                   _("delete administrative member '%s'? "),
+		                   "delete admin `%s' from shadow group `%s'",
+		                   errors) == 1) {
+			*changed = 1;
 			sge->changed = 1;
 			__sgr_set_changed ();
 		}
@@ -625,72 +716,74 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure each member exists
 		 */
-		for (i = 0; sgr->sg_mem[i]; i++) {
-			if (getpwnam (sgr->sg_mem[i]))
-				continue;
-
-			/*
-			 * Can't find this user. Remove them from the list.
-			 */
-			errors++;
-			printf (_("shadow group %s: no user %s\n"),
-				sgr->sg_name, sgr->sg_mem[i]);
-			printf (_("delete member '%s'? "), sgr->sg_mem[i]);
-
-			if (!yes_or_no ())
-				continue;
-
-			SYSLOG ((LOG_INFO,
-				 "delete member `%s' from shadow group `%s'",
-				 sgr->sg_mem[i], sgr->sg_name));
-			changed++;
-			delete_member (sgr->sg_mem, sgr->sg_mem[i]);
+		if (check_members (sgr->sg_name, sgr->sg_mem,
+		                   _("shadow group %s: no user %s\n"),
+		                   _("delete member '%s'? "),
+		                   "delete member `%s' from shadow group `%s'",
+		                   errors) == 1) {
+			*changed = 1;
 			sge->changed = 1;
 			__sgr_set_changed ();
 		}
 	}
-
-      shadow_done:
+}
 #endif				/* SHADOWGRP */
 
+/*
+ * grpck - verify group file integrity
+ */
+int main (int argc, char **argv)
+{
+	int errors = 0;
+	int changed = 0;
+
 	/*
-	 * All done. If there were no change we can just abandon any
-	 * changes to the files.
+	 * Get my name so that I can use it to report errors.
 	 */
-	if (changed) {
-	      write_and_bye:
-		if (!gr_close ()) {
-			fprintf (stderr, _("%s: cannot update file %s\n"),
-				 Prog, grp_file);
-			exit (E_CANT_UPDATE);
-		}
+	Prog = Basename (argv[0]);
+
+	setlocale (LC_ALL, "");
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+
+	OPENLOG ("grpck");
+
+	/* Parse the command line arguments */
+	process_flags (argc, argv);
+
+	open_files ();
+
+	if (sort_mode) {
+		gr_sort ();
 #ifdef	SHADOWGRP
-		if (is_shadow && !sgr_close ()) {
-			fprintf (stderr, _("%s: cannot update file %s\n"),
-				 Prog, sgr_file);
-			exit (E_CANT_UPDATE);
+		if (is_shadow) {
+			sgr_sort ();
+		}
+		changed = 1;
+#endif
+	} else {
+		check_grp_file (&errors, &changed);
+#ifdef	SHADOWGRP
+		if (is_shadow) {
+			check_sgr_file (&errors, &changed);
 		}
 #endif
 	}
 
-	/*
-	 * Don't be anti-social - unlock the files when you're done.
-	 */
-#ifdef	SHADOWGRP
-	if (is_shadow)
-		sgr_unlock ();
-#endif
-	(void) gr_unlock ();
+	/* Commit the change in the database if needed */
+	close_files (changed);
 
 	nscd_flush_cache ("group");
 
 	/*
 	 * Tell the user what we did and exit.
 	 */
-	if (errors)
+	if (errors != 0) {
 		printf (changed ?
-			_("%s: the files have been updated\n") :
-			_("%s: no changes\n"), Prog);
+		        _("%s: the files have been updated\n") :
+		        _("%s: no changes\n"), Prog);
+	}
 
 	exit (errors ? E_BAD_ENTRY : E_OKAY);
 }
+
