@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: lastlog.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: lastlog.c 1751 2008-02-03 16:28:03Z nekral-guest $"
 
 #include <getopt.h>
 #include <lastlog.h>
@@ -50,32 +50,32 @@
  * Global variables
  */
 static FILE *lastlogfile;	/* lastlog file stream */
-static off_t user;		/* one single user, specified on command line */
+static long umin;		/* if uflg, only display users with uid >= umin */
+static long umax;		/* if uflg, only display users with uid <= umax */
 static int days;		/* number of days to consider for print command */
 static time_t seconds;		/* that number of days in seconds */
 static int inverse_days;	/* number of days to consider for print command */
 static time_t inverse_seconds;	/* that number of days in seconds */
 
 
-static int uflg = 0;		/* set if user is a valid user id */
+static int uflg = 0;		/* print only an user of range of users */
 static int tflg = 0;		/* print is restricted to most recent days */
 static int bflg = 0;		/* print excludes most recent days */
 static struct lastlog lastlog;	/* scratch structure to play with ... */
-static struct stat statbuf;	/* fstat buffer for file size */
 static struct passwd *pwent;
 
 #define	NOW	(time ((time_t *) 0))
 
 static void usage (void)
 {
-	fprintf (stdout, _("Usage: lastlog [options]\n"
-			   "\n"
-			   "Options:\n"
-			   "  -b, --before DAYS             print only lastlog records older than DAYS\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -t, --time DAYS               print only lastlog records more recent than DAYS\n"
-			   "  -u, --user LOGIN              print lastlog record of the specified LOGIN\n"
-			   "\n"));
+	fputs (_("Usage: lastlog [options]\n"
+	         "\n"
+	         "Options:\n"
+	         "  -b, --before DAYS             print only lastlog records older than DAYS\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -t, --time DAYS               print only lastlog records more recent than DAYS\n"
+	         "  -u, --user LOGIN              print lastlog record of the specified LOGIN\n"
+	         "\n"), stderr);
 	exit (1);
 }
 
@@ -95,10 +95,9 @@ static void print_one (const struct passwd *pw)
 
 	if (!once) {
 #ifdef HAVE_LL_HOST
-		printf (_
-			("Username         Port     From             Latest\n"));
+		puts (_("Username         Port     From             Latest"));
 #else
-		printf (_("Username                Port     Latest\n"));
+		puts (_("Username                Port     Latest"));
 #endif
 		once++;
 	}
@@ -126,42 +125,29 @@ static void print_one (const struct passwd *pw)
 static void print (void)
 {
 	off_t offset;
+	uid_t user;
 
-	if (uflg) {
+	setpwent ();
+	while ((pwent = getpwent ())) {
+		user = pwent->pw_uid;
+		if (uflg &&
+		    ((umin != -1 && user < (uid_t)umin) ||
+		     (umax != -1 && user > (uid_t)umax)))
+			continue;
 		offset = user * sizeof lastlog;
 
-		if (fstat (fileno (lastlogfile), &statbuf)) {
-			perror (LASTLOG_FILE);
-			return;
-		}
-		if (offset >= statbuf.st_size)
-			return;
-
 		fseeko (lastlogfile, offset, SEEK_SET);
 		if (fread ((char *) &lastlog, sizeof lastlog, 1,
-			   lastlogfile) == 1)
-			print_one (pwent);
-		else
-			perror (LASTLOG_FILE);
-	} else {
-		setpwent ();
-		while ((pwent = getpwent ())) {
-			user = pwent->pw_uid;
-			offset = user * sizeof lastlog;
+			   lastlogfile) != 1)
+			continue;
 
-			fseeko (lastlogfile, offset, SEEK_SET);
-			if (fread ((char *) &lastlog, sizeof lastlog, 1,
-				   lastlogfile) != 1)
-				continue;
+		if (tflg && NOW - lastlog.ll_time > seconds)
+			continue;
 
-			if (tflg && NOW - lastlog.ll_time > seconds)
-				continue;
+		if (bflg && NOW - lastlog.ll_time < inverse_seconds)
+			continue;
 
-			if (bflg && NOW - lastlog.ll_time < inverse_seconds)
-				continue;
-
-			print_one (pwent);
-		}
+		print_one (pwent);
 	}
 }
 
@@ -199,15 +185,47 @@ int main (int argc, char **argv)
 				bflg++;
 				break;
 			case 'u':
-				pwent = xgetpwnam (optarg);
-				if (!pwent) {
-					fprintf (stderr,
-						 _("Unknown User: %s\n"),
-						 optarg);
-					exit (1);
-				}
+				/*
+				 * The user can be:
+				 *  - a login name
+				 *  - numerical
+				 *  - a numerical login ID
+				 *  - a range (-x, x-, x-y)
+				 */
 				uflg++;
-				user = pwent->pw_uid;
+				pwent = xgetpwnam (optarg);
+				if (NULL != pwent) {
+					umin = pwent->pw_uid;
+					umax = umin;
+				} else {
+					char *endptr = NULL;
+					long int user;
+					user = strtol(optarg, &endptr, 10);
+					if (*optarg != '\0' && *endptr == '\0') {
+						if (user < 0) {
+							/* -<userid> */
+							umin = -1;
+							umax = -user;
+						} else {
+							/* <userid> */
+							umin = user;
+							umax = user;
+						}
+					} else if (endptr[0] == '-' && endptr[1] == '\0') {
+						/* <userid>- */
+						umin = user;
+						umax = -1;
+					} else if (*endptr == '-') {
+						/* <userid>-<userid> */
+						umin = user;
+						umax = atol(endptr+1);
+					} else {
+						fprintf (stderr,
+							 _("Unknown user or range: %s\n"),
+							 optarg);
+						exit (1);
+					}
+				}
 				break;
 			default:
 				usage ();
