@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: login.c,v 1.86 2006/07/10 04:11:32 kloczek Exp $"
+#ident "$Id: login.c 1902 2008-03-17 23:04:46Z nekral-guest $"
 
 #include <errno.h>
 #include <grp.h>
@@ -41,6 +41,7 @@
 #include <signal.h>
 #include <stdio.h>
 #include <sys/stat.h>
+#include <sys/ioctl.h>
 #include "defines.h"
 #include "faillog.h"
 #include "failure.h"
@@ -111,10 +112,6 @@ static int timeout;
 
 extern char **newenvp;
 extern size_t newenvc;
-
-extern void dolastlog (struct lastlog *, const struct passwd *,
-		       const char *, const char *);
-
 extern char **environ;
 
 #ifndef	ALARM
@@ -131,8 +128,6 @@ static void setup_tty (void);
 static void check_flags (int, char *const *);
 
 #ifndef USE_PAM
-extern int login_access (const char *, const char *);
-
 static struct faillog faillog;
 
 static void bad_time_notify (void);
@@ -192,7 +187,7 @@ static void setup_tty (void)
  */
 static void bad_time_notify (void)
 {
-	printf (_("Invalid login time\n"));
+	puts (_("Invalid login time"));
 	fflush (stdout);
 }
 
@@ -226,7 +221,7 @@ static void check_nologin (void)
 			fflush (stdout);
 			fclose (nlfp);
 		} else
-			printf (_("\nSystem closed for routine maintenance\n"));
+			puts (_("\nSystem closed for routine maintenance"));
 		/*
 		 * Non-root users must exit. Root gets the message, but
 		 * gets to login.
@@ -236,7 +231,7 @@ static void check_nologin (void)
 			closelog ();
 			exit (0);
 		}
-		printf (_("\n[Disconnect bypassed -- root login allowed.]\n"));
+		puts (_("\n[Disconnect bypassed -- root login allowed.]"));
 	}
 }
 #endif				/* !USE_PAM */
@@ -293,7 +288,7 @@ static void init_env (void)
 }
 
 
-static RETSIGTYPE alarm_handler (int sig)
+static RETSIGTYPE alarm_handler (unused int sig)
 {
 	fprintf (stderr, _("\nLogin timed out after %d seconds.\n"), timeout);
 	exit (0);
@@ -334,7 +329,9 @@ int main (int argc, char **argv)
 	int failed;
 	int flag;
 	int subroot = 0;
+#ifndef USE_PAM
 	int is_console;
+#endif
 	int err;
 	const char *cp;
 	char *tmp;
@@ -348,6 +345,7 @@ int main (int argc, char **argv)
 	int retcode;
 	pid_t child;
 	char *pam_user;
+	char **ptr_pam_user = &pam_user;
 #else
 	struct spwd *spwd = NULL;
 #endif
@@ -439,7 +437,9 @@ int main (int argc, char **argv)
 	 */
 	checkutmp (!amroot);
 	STRFCPY (tty, utent.ut_line);
+#ifndef USE_PAM
 	is_console = console (tty);
+#endif
 
 	if (rflg || hflg) {
 #ifdef UT_ADDR
@@ -558,10 +558,10 @@ int main (int argc, char **argv)
 
 		if (*cp)
 			snprintf (fromhost, sizeof fromhost,
-				  _(" on '%.100s' from '%.200s'"), tty, cp);
+			          " on '%.100s' from '%.200s'", tty, cp);
 		else
 			snprintf (fromhost, sizeof fromhost,
-				  _(" on '%.100s'"), tty);
+			          " on '%.100s'", tty);
 
 	      top:
 		/* only allow ALARM sec. for login */
@@ -602,25 +602,25 @@ int main (int argc, char **argv)
 		if (!fflg || (getuid () != 0)) {
 			int failcount = 0;
 			char hostn[256];
-			char login_prompt[256];	/* That's one hell of a prompt :) */
+			char loginprompt[256];	/* That's one hell of a prompt :) */
 
 			/* Make the login prompt look like we want it */
 			if (!gethostname (hostn, sizeof (hostn)))
-				snprintf (login_prompt,
-					  sizeof (login_prompt),
+				snprintf (loginprompt,
+					  sizeof (loginprompt),
 					  _("%s login: "), hostn);
 			else
-				snprintf (login_prompt,
-					  sizeof (login_prompt), _("login: "));
+				snprintf (loginprompt,
+					  sizeof (loginprompt), _("login: "));
 
 			retcode =
-			    pam_set_item (pamh, PAM_USER_PROMPT, login_prompt);
+			    pam_set_item (pamh, PAM_USER_PROMPT, loginprompt);
 			PAM_FAIL_CHECK;
 
 			/* if we didn't get a user on the command line,
 			   set it to NULL */
 			pam_get_item (pamh, PAM_USER,
-				      (const void **) &pam_user);
+				      (const void **)ptr_pam_user);
 			if (pam_user[0] == '\0')
 				pam_set_item (pamh, PAM_USER, NULL);
 
@@ -644,10 +644,10 @@ int main (int argc, char **argv)
 			  retcode = pam_authenticate (pamh, 0);
 
 			  pam_get_item (pamh, PAM_USER,
-					(const void **) &pam_user);
+					(const void **) ptr_pam_user);
 
 			  if (pam_user && pam_user[0]) {
-			    pwd = getpwnam(pam_user);
+			    pwd = xgetpwnam(pam_user);
 			    if (pwd) {
 			      pwent = *pwd;
 			      failent_user = pwent.pw_name;
@@ -673,7 +673,7 @@ int main (int argc, char **argv)
 			    exit(0);
 			  } else if (retcode == PAM_ABORT) {
 			    /* Serious problems, quit now */
-			    fprintf(stderr,_("login: abort requested by PAM\n"));
+			    fputs (_("login: abort requested by PAM\n"),stderr);
 			    SYSLOG ((LOG_ERR,"PAM_ABORT returned from pam_authenticate()"));
 			    PAM_END;
 			    exit(99);
@@ -693,6 +693,7 @@ int main (int argc, char **argv)
 					char buf[64];
 
 					audit_fd = audit_open ();
+					/* local, no need for xgetpwnam */
 					pw = getpwnam (username);
 					if (pw) {
 						snprintf (buf, sizeof (buf),
@@ -736,19 +737,25 @@ int main (int argc, char **argv)
 		   First get the username that we are actually using, though.
 		 */
 		retcode =
-		    pam_get_item (pamh, PAM_USER, (const void **) &pam_user);
+		    pam_get_item (pamh, PAM_USER, (const void **)ptr_pam_user);
 		setpwent ();
-		pwd = getpwnam (pam_user);
+		pwd = xgetpwnam (pam_user);
+		if (!pwd) {
+			SYSLOG ((LOG_ERR, "xgetpwnam(%s) failed",
+				 getdef_bool ("LOG_UNKFAIL_ENAB") ?
+				 pam_user : "UNKNOWN"));
+			exit (1);
+		}
 
 		if (fflg) {
 			retcode = pam_acct_mgmt (pamh, 0);
 			PAM_FAIL_CHECK;
 		}
 
-		if (!pwd || setup_groups (pwd))
+		if (setup_groups (pwd))
 			exit (1);
-		else
-			pwent = *pwd;
+
+		pwent = *pwd;
 
 		retcode = pam_setcred (pamh, PAM_ESTABLISH_CRED);
 		PAM_FAIL_CHECK;
@@ -773,10 +780,10 @@ int main (int argc, char **argv)
 #endif				/* ! USE_PAM */
 
 #ifdef USE_PAM
-		if (!(pwd = getpwnam (pam_user))) {
+		if (!(pwd = xgetpwnam (pam_user))) {
 			pwent.pw_name = pam_user;
 #else
-		if (!(pwd = getpwnam (username))) {
+		if (!(pwd = xgetpwnam (username))) {
 			pwent.pw_name = username;
 #endif
 			strcpy (temp_pw, "!");
@@ -791,6 +798,7 @@ int main (int argc, char **argv)
 #ifndef USE_PAM
 		spwd = NULL;
 		if (pwd && strcmp (pwd->pw_passwd, SHADOW_PASSWD_STRING) == 0) {
+			/* !USE_PAM, no need for xgetspnam */
 			spwd = getspnam (username);
 			if (spwd)
 				pwent.pw_passwd = spwd->sp_pwdp;
@@ -990,7 +998,9 @@ int main (int argc, char **argv)
 	 */
 	if (spwd) {		/* check for age of password */
 		if (expire (&pwent, spwd)) {
+			/* !USE_PAM, no need for xgetpwnam */
 			pwd = getpwnam (username);
+			/* !USE_PAM, no need for xgetspnam */
 			spwd = getspnam (username);
 			if (pwd)
 				pwent = *pwd;
@@ -1024,6 +1034,12 @@ int main (int argc, char **argv)
 	}
 	/* child */
 #endif
+	/* If we were init, we need to start a new session */
+	if (getppid() == 1) {
+		setsid();
+		if (ioctl(0, TIOCSCTTY, 1))
+			fprintf(stderr,_("TIOCSCTTY failed on %s"),tty);
+	}
 
 	/* We call set_groups() above because this clobbers pam_groups.so */
 #ifndef USE_PAM
@@ -1066,7 +1082,7 @@ int main (int argc, char **argv)
 			if (faillog.fail_max &&
 			    faillog.fail_cnt >= faillog.fail_max) {
 				puts (_
-				      ("Warning: login re-enabled after temporary lockout.\n"));
+				      ("Warning: login re-enabled after temporary lockout."));
 				SYSLOG ((LOG_WARN,
 					 "login `%s' re-enabled after temporary lockout (%d failures)",
 					 username, (int) faillog.fail_cnt));
