@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: newgrp.c,v 1.44 2006/01/18 19:55:15 kloczek Exp $"
+#ident "$Id: newgrp.c 1876 2008-03-07 20:21:15Z nekral-guest $"
 
 #include <errno.h>
 #include <grp.h>
@@ -55,6 +55,10 @@ static int is_newgrp;
 
 /* local function prototypes */
 static void usage (void);
+static void check_perms (const struct group *grp,
+                         struct passwd *pwd,
+                         const char *groupname);
+static void syslog_sg (const char *name, const char *group);
 
 /*
  * usage - print command usage message
@@ -62,9 +66,9 @@ static void usage (void);
 static void usage (void)
 {
 	if (is_newgrp)
-		fprintf (stderr, _("Usage: newgrp [-] [group]\n"));
+		fputs (_("Usage: newgrp [-] [group]\n"), stderr);
 	else
-		fprintf (stderr, _("Usage: sg group [[-c] command]\n"));
+		fputs (_("Usage: sg group [[-c] command]\n"), stderr);
 }
 
 /*
@@ -96,32 +100,257 @@ static struct group *find_matching_group (const char *name, gid_t gid)
 	return gr;
 }
 
+/*
+ * check_perms - check if the user is allowed to switch to this group
+ *
+ *	If needed, the user will be authenticated.
+ *
+ *	It will not return if the user could not be authenticated.
+ */
+static void check_perms (const struct group *grp,
+                         struct passwd *pwd,
+                         const char *groupname)
+{
+	int needspasswd = 0;
+	struct spwd *spwd;
+	char *cp;
+	const char *cpasswd;
+
+	/*
+	 * see if she is a member of this group (i.e. in the list of
+	 * members of the group, or if the group is her primary group).
+	 *
+	 * If she isn't a member, she needs to provide the group password.
+	 * If there is no group password, she will be denied access
+	 * anyway.
+	 *
+	 */
+	if (grp->gr_gid != pwd->pw_gid && !is_on_list (grp->gr_mem, pwd->pw_name))
+		needspasswd = 1;
+
+	/*
+	 * If she does not have either a shadowed password, or a regular
+	 * password, and the group has a password, she needs to give the
+	 * group password.
+	 */
+	spwd = xgetspnam (pwd->pw_name);
+	if (NULL != spwd)
+		pwd->pw_passwd = spwd->sp_pwdp;
+
+	if (pwd->pw_passwd[0] == '\0' && grp->gr_passwd[0])
+		needspasswd = 1;
+
+	/*
+	 * Now I see about letting her into the group she requested. If she
+	 * is the root user, I'll let her in without having to prompt for
+	 * the password. Otherwise I ask for a password if she flunked one
+	 * of the tests above.
+	 */
+	if (getuid () != 0 && needspasswd) {
+		/*
+		 * get the password from her, and set the salt for
+		 * the decryption from the group file.
+		 */
+		cp = getpass (_("Password: "));
+		if (NULL == cp)
+			goto failure;
+
+		/*
+		 * encrypt the key she gave us using the salt from the
+		 * password in the group file. The result of this encryption
+		 * must match the previously encrypted value in the file.
+		 */
+		cpasswd = pw_encrypt (cp, grp->gr_passwd);
+		strzero (cp);
+
+		if (grp->gr_passwd[0] == '\0' ||
+		    strcmp (cpasswd, grp->gr_passwd) != 0) {
+#ifdef WITH_AUDIT
+			snprintf (audit_buf, sizeof(audit_buf),
+			          "authentication new-gid=%d", grp->gr_gid);
+			audit_logger (AUDIT_GRP_AUTH, Prog,
+			              audit_buf, NULL, getuid (), 0);
+#endif
+			SYSLOG ((LOG_INFO,
+				 "Invalid password for group `%s' from `%s'",
+				 groupname, pwd->pw_name));
+			sleep (1);
+			fputs (_("Invalid password.\n"), stderr);
+			goto failure;
+		}
+#ifdef WITH_AUDIT
+		snprintf (audit_buf, sizeof(audit_buf),
+		          "authentication new-gid=%d", grp->gr_gid);
+		audit_logger (AUDIT_GRP_AUTH, Prog,
+		              audit_buf, NULL, getuid (), 1);
+#endif
+	}
+
+	return;
+
+failure:
+	/* The closelog is probably unnecessary, but it does no
+	 * harm.  -- JWP
+	 */
+	closelog ();
+#ifdef WITH_AUDIT
+	if (groupname) {
+		snprintf (audit_buf, sizeof(audit_buf),
+		          "changing new-group=%s", groupname);
+		audit_logger (AUDIT_CHGRP_ID, Prog,
+		              audit_buf, NULL, getuid (), 0);
+	} else {
+		audit_logger (AUDIT_CHGRP_ID, Prog, "changing",
+		              NULL, getuid (), 0);
+	}
+#endif
+	exit (1);
+}
+
+#ifdef USE_SYSLOG
+/*
+ * syslog_sg - log the change of group to syslog
+ *
+ *	The loggout will also be logged when the user will quit the
+ *	sg/newgrp session.
+ */
+static void syslog_sg (const char *name, const char *group)
+{
+	const char *loginname = getlogin ();
+	const char *tty = ttyname (0);
+
+	if (loginname != NULL)
+		loginname = xstrdup (loginname);
+	if (tty != NULL)
+		tty = xstrdup (tty);
+
+	if (loginname == NULL)
+		loginname = "???";
+	if (tty == NULL)
+		tty = "???";
+	else if (strncmp (tty, "/dev/", 5) == 0)
+		tty += 5;
+	SYSLOG ((LOG_INFO,
+		 "user `%s' (login `%s' on %s) switched to group `%s'",
+		 name, loginname, tty, group));
+#ifdef USE_PAM
+	/*
+	 * We want to fork and exec the new shell in the child, leaving the
+	 * parent waiting to log the session close.
+	 *
+	 * The parent must ignore signals generated from the console
+	 * (SIGINT, SIGQUIT, SIGHUP) which might make the parent terminate
+	 * before its child. When bash is exec'ed as the subshell, it
+	 * generates a new process group id for itself, and consequently
+	 * only SIGHUP, which is sent to all process groups in the session,
+	 * can reach the parent. However, since arbitrary programs can be
+	 * specified as login shells, there is no such guarantee in general.
+	 * For the same reason, we must also ignore stop signals generated
+	 * from the console (SIGTSTP, SIGTTIN, and SIGTTOU) in order to
+	 * avoid any possibility of the parent being stopped when it
+	 * receives SIGCHLD from the terminating subshell.  -- JWP
+	 */
+	{
+		pid_t child, pid;
+
+		signal (SIGINT, SIG_IGN);
+		signal (SIGQUIT, SIG_IGN);
+		signal (SIGHUP, SIG_IGN);
+		signal (SIGTSTP, SIG_IGN);
+		signal (SIGTTIN, SIG_IGN);
+		signal (SIGTTOU, SIG_IGN);
+		child = fork ();
+		if (child < 0) {
+			/* error in fork() */
+			fprintf (stderr, _("%s: failure forking: %s\n"),
+				 is_newgrp ? "newgrp" : "sg", strerror (errno));
+#ifdef WITH_AUDIT
+			if (group) {
+				snprintf (audit_buf, sizeof(audit_buf),
+				          "changing new-group=%s", group);
+				audit_logger (AUDIT_CHGRP_ID, Prog,
+				              audit_buf, NULL, getuid (), 0);
+			} else {
+				audit_logger (AUDIT_CHGRP_ID, Prog, "changing",
+				              NULL, getuid (), 0);
+			}
+#endif
+			exit (1);
+		} else if (child) {
+			/* parent - wait for child to finish, then log session close */
+			int cst = 0;
+			gid_t gid = getgid();
+			struct group *grp = getgrgid (gid);
+
+			do {
+				errno = 0;
+				pid = waitpid (child, &cst, WUNTRACED);
+				if (pid == child && WIFSTOPPED (cst)) {
+					/* stop when child stops */
+					raise (SIGSTOP);
+					/* wake child when resumed */
+					kill (child, SIGCONT);
+				}
+			} while ((pid == child && WIFSTOPPED (cst)) ||
+				 (pid != child && errno == EINTR));
+			/* local, no need for xgetgrgid */
+			if (NULL != grp) {
+				SYSLOG ((LOG_INFO,
+				         "user `%s' (login `%s' on %s) returned to group `%s'",
+				         name, loginname, tty, grp->gr_name));
+			} else {
+				SYSLOG ((LOG_INFO,
+				         "user `%s' (login `%s' on %s) returned to group `%d'",
+				         name, loginname, tty, gid));
+				/* Either the user's passwd entry has a
+				 * GID that does not match with any group,
+				 * or the group was deleted while the user
+				 * was in a newgrp session.*/
+				SYSLOG ((LOG_WARN,
+				         "unknown GID `%u' used by user `%s'",
+				         gid, name));
+			}
+			closelog ();
+			exit (0);
+		}
+
+		/* child - restore signals to their default state */
+		signal (SIGINT, SIG_DFL);
+		signal (SIGQUIT, SIG_DFL);
+		signal (SIGHUP, SIG_DFL);
+		signal (SIGTSTP, SIG_DFL);
+		signal (SIGTTIN, SIG_DFL);
+		signal (SIGTTOU, SIG_DFL);
+	}
+#endif				/* USE_PAM */
+}
+#endif				/* USE_SYSLOG */
+
 /*
  * newgrp - change the invokers current real and effective group id
  */
 int main (int argc, char **argv)
 {
 	int initflag = 0;
-	int needspasswd = 0;
 	int i;
 	int cflag = 0;
 	int err = 0;
 	gid_t gid;
 	char *cp;
-	const char *cpasswd, *name, *prog;
+	const char *name, *prog;
 	char *group = NULL;
 	char *command = NULL;
 	char **envp = environ;
 	struct passwd *pwd;
 	struct group *grp;
 
-	struct spwd *spwd;
-
 #ifdef SHADOWGRP
 	struct sgrp *sgrp;
 #endif
 
 #ifdef WITH_AUDIT
+	char audit_buf[80];
+
 	audit_help_open ();
 #endif
 	setlocale (LC_ALL, "");
@@ -164,7 +393,7 @@ int main (int argc, char **argv)
 	if (!pwd) {
 		fprintf (stderr, _("unknown UID: %u\n"), getuid ());
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_START, Prog, "changing", NULL,
+		audit_logger (AUDIT_CHGRP_ID, Prog, "changing", NULL,
 			      getuid (), 0);
 #endif
 		SYSLOG ((LOG_WARN, "unknown UID %u", getuid ()));
@@ -240,7 +469,8 @@ int main (int argc, char **argv)
 			 * Perhaps in the past, but the default behavior now depends on the
 			 * group entry, so it had better exist.  -- JWP
 			 */
-			if (!(grp = getgrgid (pwd->pw_gid))) {
+			grp = xgetgrgid (pwd->pw_gid);
+			if (NULL == grp) {
 				fprintf (stderr, _("unknown GID: %lu\n"),
 					 (unsigned long) pwd->pw_gid);
 				SYSLOG ((LOG_CRIT, "unknown GID: %lu",
@@ -272,8 +502,15 @@ int main (int argc, char **argv)
 	if (ngroups < 0) {
 		perror ("getgroups");
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_START, Prog,
-			      "changing", NULL, getuid (), 0);
+		if (group) {
+			snprintf (audit_buf, sizeof(audit_buf),
+			          "changing new-group=%s", group);
+			audit_logger (AUDIT_CHGRP_ID, Prog,
+			              audit_buf, NULL, getuid (), 0);
+		} else {
+			audit_logger (AUDIT_CHGRP_ID, Prog,
+			              "changing", NULL, getuid (), 0);
+		}
 #endif
 		exit (1);
 	}
@@ -319,7 +556,8 @@ int main (int argc, char **argv)
 	 * including the user's name in the member list of the user's login
 	 * group.  -- JWP
 	 */
-	if (!(grp = getgrnam (group))) {
+	grp = getgrnam (group); /* local, no need for xgetgrnam */
+	if (NULL == grp) {
 		fprintf (stderr, _("unknown group: %s\n"), group);
 		goto failure;
 	}
@@ -338,170 +576,28 @@ int main (int argc, char **argv)
 		 *
 		 * Re-read the group entry for further processing.
 		 */
-		grp = getgrnam (group);
+		grp = xgetgrnam (group);
 	}
 #ifdef SHADOWGRP
-	if ((sgrp = getsgnam (group))) {
+	sgrp = getsgnam (group);
+	if (NULL != sgrp) {
 		grp->gr_passwd = sgrp->sg_passwd;
 		grp->gr_mem = sgrp->sg_mem;
 	}
 #endif
 
 	/*
-	 * see if she is a member of this group. If she isn't a member, she
-	 * needs to provide the group password. If there is no group
-	 * password, she will be denied access anyway.
-	 *
+	 * Check if the user is allowed to access this group.
 	 */
-	if (!is_on_list (grp->gr_mem, name))
-		needspasswd = 1;
-
-	/*
-	 * If she does not have either a shadowed password, or a regular
-	 * password, and the group has a password, she needs to give the
-	 * group password.
-	 */
-	if ((spwd = getspnam (name)))
-		pwd->pw_passwd = spwd->sp_pwdp;
-
-	if (pwd->pw_passwd[0] == '\0' && grp->gr_passwd[0])
-		needspasswd = 1;
-
-	/*
-	 * Now i see about letting her into the group she requested. If she
-	 * is the root user, i'll let her in without having to prompt for
-	 * the password. Otherwise i ask for a password if she flunked one
-	 * of the tests above.
-	 *
-	 * Note that she now has to provide the password to her own group,
-	 * unless she is listed as a member.  -- JWP
-	 */
-	if (getuid () != 0 && needspasswd) {
-		/*
-		 * get the password from her, and set the salt for
-		 * the decryption from the group file.
-		 */
-		if (!(cp = getpass (_("Password: "))))
-			goto failure;
-
-		/*
-		 * encrypt the key she gave us using the salt from the
-		 * password in the group file. The result of this encryption
-		 * must match the previously encrypted value in the file.
-		 */
-		cpasswd = pw_encrypt (cp, grp->gr_passwd);
-		strzero (cp);
-
-		if (grp->gr_passwd[0] == '\0') {
-			/*
-			 * there is no password, print out "Sorry" and give up
-			 */
-			sleep (1);
-			fputs (_("No password.\n"), stderr);
-			goto failure;
-		}
-
-		if (strcmp (cpasswd, grp->gr_passwd) != 0) {
-			SYSLOG ((LOG_INFO,
-				 "Invalid password for group `%s' from `%s'",
-				 group, name));
-			sleep (1);
-			goto failure;
-		}
-	}
+	check_perms (grp, pwd, group);
 
 	/*
 	 * all successful validations pass through this point. The group id
 	 * will be set, and the group added to the concurrent groupset.
 	 */
 #ifdef	USE_SYSLOG
-	if (getdef_bool ("SYSLOG_SG_ENAB"))
-		SYSLOG ((LOG_INFO, "user `%s' switched to group `%s'",
-			 name, group));
 	if (getdef_bool ("SYSLOG_SG_ENAB")) {
-		char *loginname = getlogin ();
-		char *tty = ttyname (0);
-
-		if (loginname != NULL)
-			loginname = xstrdup (loginname);
-		if (tty != NULL)
-			tty = xstrdup (tty);
-
-		if (loginname == NULL)
-			loginname = "???";
-		if (tty == NULL)
-			tty = "???";
-		else if (strncmp (tty, "/dev/", 5) == 0)
-			tty += 5;
-		SYSLOG ((LOG_INFO,
-			 "user `%s' (login `%s' on %s) switched to group `%s'",
-			 name, loginname, tty, group));
-#ifdef USE_PAM
-		/*
-		 * We want to fork and exec the new shell in the child, leaving the
-		 * parent waiting to log the session close.
-		 *
-		 * The parent must ignore signals generated from the console
-		 * (SIGINT, SIGQUIT, SIGHUP) which might make the parent terminate
-		 * before its child. When bash is exec'ed as the subshell, it
-		 * generates a new process group id for itself, and consequently
-		 * only SIGHUP, which is sent to all process groups in the session,
-		 * can reach the parent. However, since arbitrary programs can be
-		 * specified as login shells, there is no such guarantee in general.
-		 * For the same reason, we must also ignore stop signals generated
-		 * from the console (SIGTSTP, SIGTTIN, and SIGTTOU) in order to
-		 * avoid any possibility of the parent being stopped when it
-		 * receives SIGCHLD from the terminating subshell.  -- JWP
-		 */
-		pid_t child, pid;
-
-		signal (SIGINT, SIG_IGN);
-		signal (SIGQUIT, SIG_IGN);
-		signal (SIGHUP, SIG_IGN);
-		signal (SIGTSTP, SIG_IGN);
-		signal (SIGTTIN, SIG_IGN);
-		signal (SIGTTOU, SIG_IGN);
-		child = fork ();
-		if (child < 0) {
-			/* error in fork() */
-			fprintf (stderr, _("%s: failure forking: %s"),
-				 is_newgrp ? "newgrp" : "sg", strerror (errno));
-#ifdef WITH_AUDIT
-			audit_logger (AUDIT_USER_START, Prog, "changing",
-				      NULL, getuid (), 0);
-#endif
-			exit (1);
-		} else if (child) {
-			/* parent - wait for child to finish, then log session close */
-			int cst = 0;
-
-			do {
-				errno = 0;
-				pid = waitpid (child, &cst, WUNTRACED);
-				if (pid == child && WIFSTOPPED (cst)) {
-					/* stop when child stops */
-					raise (SIGSTOP);
-					/* wake child when resumed */
-					kill (child, SIGCONT);
-				}
-			} while ((pid == child && WIFSTOPPED (cst)) ||
-				 (pid != child && errno == EINTR));
-			SYSLOG ((LOG_INFO,
-				 "user `%s' (login `%s' on %s) returned to group `%s'",
-				 name, loginname, tty,
-				 getgrgid (gid)->gr_name));
-			closelog ();
-			exit (0);
-		}
-
-		/* child - restore signals to their default state */
-		signal (SIGINT, SIG_DFL);
-		signal (SIGQUIT, SIG_DFL);
-		signal (SIGHUP, SIG_DFL);
-		signal (SIGTSTP, SIG_DFL);
-		signal (SIGTTIN, SIG_DFL);
-		signal (SIGTTOU, SIG_DFL);
-#endif				/* USE_PAM */
+		syslog_sg (name, group);
 	}
 #endif				/* USE_SYSLOG */
 
@@ -520,7 +616,7 @@ int main (int argc, char **argv)
 	}
 	if (i == ngroups) {
 		if (ngroups >= sysconf (_SC_NGROUPS_MAX)) {
-			fprintf (stderr, _("too many groups\n"));
+			fputs (_("too many groups\n"), stderr);
 		} else {
 			grouplist[ngroups++] = gid;
 			if (setgroups (ngroups, grouplist)) {
@@ -535,14 +631,24 @@ int main (int argc, char **argv)
 	 * to the real UID. For root, this also sets the real GID to the
 	 * new group id.
 	 */
-	if (setgid (gid))
+	if (setgid (gid)) {
 		perror ("setgid");
+#ifdef WITH_AUDIT
+		snprintf (audit_buf, sizeof(audit_buf),
+		          "changing new-gid=%d", gid);
+		audit_logger (AUDIT_CHGRP_ID, Prog,
+		              audit_buf, NULL, getuid (), 0);
+#endif
+		exit (1);
+	}
 
 	if (setuid (getuid ())) {
 		perror ("setuid");
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_START, Prog, "changing",
-			      NULL, getuid (), 0);
+		snprintf (audit_buf, sizeof(audit_buf),
+		          "changing new-gid=%d", gid);
+		audit_logger (AUDIT_CHGRP_ID, Prog,
+		              audit_buf, NULL, getuid (), 0);
 #endif
 		exit (1);
 	}
@@ -555,8 +661,10 @@ int main (int argc, char **argv)
 		closelog ();
 		execl ("/bin/sh", "sh", "-c", command, (char *) 0);
 #ifdef WITH_AUDIT
-		audit_logger (AUDIT_USER_START, Prog, "changing",
-			      NULL, getuid (), 0);
+		snprintf (audit_buf, sizeof(audit_buf),
+		          "changing new-gid=%d", gid);
+		audit_logger (AUDIT_CHGRP_ID, Prog,
+		              audit_buf, NULL, getuid (), 0);
 #endif
 		perror ("/bin/sh");
 		exit (errno == ENOENT ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
@@ -622,7 +730,8 @@ int main (int argc, char **argv)
 	}
 
 #ifdef WITH_AUDIT
-	audit_logger (AUDIT_USER_START, Prog, "changing", NULL, getuid (), 1);
+	snprintf (audit_buf, sizeof(audit_buf), "changing new-gid=%d", gid);
+	audit_logger (AUDIT_CHGRP_ID, Prog, audit_buf, NULL, getuid (), 1);
 #endif
 	/*
 	 * Exec the login shell and go away. We are trying to get back to
@@ -645,7 +754,15 @@ int main (int argc, char **argv)
 	 */
 	closelog ();
 #ifdef WITH_AUDIT
-	audit_logger (AUDIT_USER_START, Prog, "changing", NULL, getuid (), 0);
+	if (group) {
+		snprintf (audit_buf, sizeof(audit_buf),
+		          "changing new-group=%s", group);
+		audit_logger (AUDIT_CHGRP_ID, Prog, 
+		              audit_buf, NULL, getuid (), 0);
+	} else {
+		audit_logger (AUDIT_CHGRP_ID, Prog,
+		              "changing", NULL, getuid (), 0);
+	}
 #endif
 	exit (1);
 }
