@@ -23,14 +23,14 @@
 
 #include <config.h>
 
-#ident "$Id: nologin.c 1407 2007-11-19 20:25:36Z nekral-guest $"
+#ident "$Id: nologin.c 1639 2008-01-05 13:51:43Z nekral-guest $"
 
 #include <stdio.h>
 #include <syslog.h>
 #include <unistd.h>
 #include "exitcodes.h"
 
-int main ()
+int main (void)
 {
 	const char *user, *tty;
 
