@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: passwd.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: passwd.c 1932 2008-03-26 22:00:50Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -40,7 +40,9 @@
 #include <sys/types.h>
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
+#include <selinux/flask.h>
 #include <selinux/av_permissions.h>
+#include <selinux/context.h>
 #endif
 #include <time.h>
 #include "defines.h"
@@ -143,27 +145,27 @@ static long getnumber (const char *);
  */
 static void usage (int status)
 {
-	fprintf (stderr, _("Usage: passwd [options] [LOGIN]\n"
-			   "\n"
-			   "Options:\n"
-			   "  -a, --all                     report password status on all accounts\n"
-			   "  -d, --delete                  delete the password for the named account\n"
-			   "  -e, --expire                  force expire the password for the named account\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -k, --keep-tokens             change password only if expired\n"
-			   "  -i, --inactive INACTIVE       set password inactive after expiration\n"
-			   "                                to INACTIVE\n"
-			   "  -l, --lock                    lock the named account\n"
-			   "  -n, --mindays MIN_DAYS        set minimum number of days before password\n"
-			   "                                change to MIN_DAYS\n"
-			   "  -q, --quiet                   quiet mode\n"
-			   "  -r, --repository REPOSITORY   change password in REPOSITORY repository\n"
-			   "  -S, --status                  report password status on the named account\n"
-			   "  -u, --unlock                  unlock the named account\n"
-			   "  -w, --warndays WARN_DAYS      set expiration warning days to WARN_DAYS\n"
-			   "  -x, --maxdays MAX_DAYS        set maximim number of days before password\n"
-			   "                                change to MAX_DAYS\n"
-			   "\n"));
+	fputs (_("Usage: passwd [options] [LOGIN]\n"
+	         "\n"
+	         "Options:\n"
+	         "  -a, --all                     report password status on all accounts\n"
+	         "  -d, --delete                  delete the password for the named account\n"
+	         "  -e, --expire                  force expire the password for the named account\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -k, --keep-tokens             change password only if expired\n"
+	         "  -i, --inactive INACTIVE       set password inactive after expiration\n"
+	         "                                to INACTIVE\n"
+	         "  -l, --lock                    lock the named account\n"
+	         "  -n, --mindays MIN_DAYS        set minimum number of days before password\n"
+	         "                                change to MIN_DAYS\n"
+	         "  -q, --quiet                   quiet mode\n"
+	         "  -r, --repository REPOSITORY   change password in REPOSITORY repository\n"
+	         "  -S, --status                  report password status on the named account\n"
+	         "  -u, --unlock                  unlock the named account\n"
+	         "  -w, --warndays WARN_DAYS      set expiration warning days to WARN_DAYS\n"
+	         "  -x, --maxdays MAX_DAYS        set maximim number of days before password\n"
+	         "                                change to MAX_DAYS\n"
+	         "\n"), stderr);
 	exit (status);
 }
 
@@ -282,7 +284,7 @@ static int new_password (const struct passwd *pw)
 		strzero (cp);
 
 		if (!amroot && (!obscure (orig, pass, pw) || reuse (pass, pw))) {
-			printf (_("Try again.\n"));
+			puts (_("Try again."));
 			continue;
 		}
 
@@ -293,8 +295,7 @@ static int new_password (const struct passwd *pw)
 		 */
 		if (amroot && !warned && getdef_bool ("PASS_ALWAYS_WARN")
 		    && (!obscure (orig, pass, pw) || reuse (pass, pw))) {
-			printf (_
-				("\nWarning: weak password (enter it again to use it anyway).\n"));
+			puts (_("\nWarning: weak password (enter it again to use it anyway)."));
 			warned++;
 			continue;
 		}
@@ -303,7 +304,7 @@ static int new_password (const struct passwd *pw)
 			return -1;
 		}
 		if (strcmp (cp, pass))
-			fprintf (stderr, _("They don't match; try again.\n"));
+			fputs (_("They don't match; try again.\n"), stderr);
 		else {
 			strzero (cp);
 			break;
@@ -468,7 +469,7 @@ static char *update_crypt_pw (char *cp)
 #endif
 
 	if (dflg)
-		cp = "";	/* XXX warning: const */
+		*cp = '\0';
 
 	if (uflg && *cp == '!') {
 		if (cp[1] == '\0') {
@@ -498,14 +499,13 @@ static void update_noshadow (void)
 	struct passwd *npw;
 
 	if (!pw_lock ()) {
-		fprintf (stderr,
-			 _
-			 ("Cannot lock the password file; try again later.\n"));
+		fputs (_("Cannot lock the password file; try again later.\n"),
+		       stderr);
 		SYSLOG ((LOG_WARN, "can't lock password file"));
 		exit (E_PWDBUSY);
 	}
 	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("Cannot open the password file.\n"));
+		fputs (_("Cannot open the password file.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't open password file"));
 		fail_exit (E_MISSING);
 	}
@@ -520,12 +520,12 @@ static void update_noshadow (void)
 		oom ();
 	npw->pw_passwd = update_crypt_pw (npw->pw_passwd);
 	if (!pw_update (npw)) {
-		fprintf (stderr, _("Error updating the password entry.\n"));
+		fputs (_("Error updating the password entry.\n"), stderr);
 		SYSLOG ((LOG_ERR, "error updating password entry"));
 		fail_exit (E_FAILURE);
 	}
 	if (!pw_close ()) {
-		fprintf (stderr, _("Cannot commit password file changes.\n"));
+		fputs (_("Cannot commit password file changes.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't rewrite password file"));
 		fail_exit (E_FAILURE);
 	}
@@ -538,14 +538,13 @@ static void update_shadow (void)
 	struct spwd *nsp;
 
 	if (!spw_lock ()) {
-		fprintf (stderr,
-			 _
-			 ("Cannot lock the password file; try again later.\n"));
+		fputs (_("Cannot lock the password file; try again later.\n"),
+		       stderr);
 		SYSLOG ((LOG_WARN, "can't lock password file"));
 		exit (E_PWDBUSY);
 	}
 	if (!spw_open (O_RDWR)) {
-		fprintf (stderr, _("Cannot open the password file.\n"));
+		fputs (_("Cannot open the password file.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't open password file"));
 		fail_exit (E_FAILURE);
 	}
@@ -590,12 +589,12 @@ static void update_shadow (void)
 		nsp->sp_lstchg = 0;
 
 	if (!spw_update (nsp)) {
-		fprintf (stderr, _("Error updating the password entry.\n"));
+		fputs (_("Error updating the password entry.\n"), stderr);
 		SYSLOG ((LOG_ERR, "error updating password entry"));
 		fail_exit (E_FAILURE);
 	}
 	if (!spw_close ()) {
-		fprintf (stderr, _("Cannot commit password file changes.\n"));
+		fputs (_("Cannot commit password file changes.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't rewrite password file"));
 		fail_exit (E_FAILURE);
 	}
@@ -616,6 +615,49 @@ static long getnumber (const char *numstr)
 	return val;
 }
 
+#ifdef WITH_SELINUX
+int
+check_selinux_access(const char *change_user, int change_uid, unsigned int access)
+{
+	int status = -1;
+	security_context_t user_context;
+	const char *user;
+
+	/* if in permissive mode then allow the operation */
+	if (security_getenforce() == 0)
+		return 0;
+
+	/* get the context of the process which executed passwd */
+	if (getprevcon(&user_context))
+		return -1;
+
+	/* get the "user" portion of the context (the part before the first
+	   colon) */
+	context_t c;
+	c = context_new(user_context);
+	user = context_user_get(c);
+
+	/* if changing a password for an account with UID==0 or for an account
+	   where the identity matches then return success */
+	if (change_uid != 0 && strcmp(change_user, user) == 0) {
+		status = 0;
+	} else {
+		struct av_decision avd;
+		int retval;
+		retval = security_compute_av(user_context, user_context,
+				SECCLASS_PASSWD, access, &avd);
+		if ((retval == 0) &&
+    			((access & avd.allowed) == access)) {
+			status = 0;
+		}
+	}
+	context_free(c);
+	freecon(user_context);
+	return status;
+}
+
+#endif
+
 /*
  * passwd - change a user's password file information
  *
@@ -788,6 +830,12 @@ int main (int argc, char **argv)
 	else
 		name = myname;
 
+	/*
+	 * Make sure that at most one username was specified.
+	 */
+	if (argc > optind+1)
+		usage (E_USAGE);
+
 	/*
 	 * The -a flag requires -S, no other flags, no username, and
 	 * you must be root.  --marekm
@@ -841,21 +889,32 @@ int main (int argc, char **argv)
 		exit (E_NOPERM);
 	}
 #ifdef WITH_SELINUX
-	/*
-	 * If the UID of the user does not match the current real UID,
-	 * check if the change is allowed by SELinux policy.
-	 */
-	if ((pw->pw_uid != getuid ())
-	    && (is_selinux_enabled () > 0 ?
-		(selinux_check_passwd_access (PASSWD__PASSWD) != 0) :
-		!amroot)) {
-#else
+	/* only do this check when getuid()==0 because it's a pre-condition for
+	   changing a password without entering the old one */
+	if ((is_selinux_enabled() > 0) && (getuid() == 0) &&
+	  (check_selinux_access(name, pw->pw_uid, PASSWD__PASSWD) != 0))
+	{
+		security_context_t user_context;
+		if (getprevcon(&user_context) < 0) {
+			user_context = strdup("Unknown user context");
+		}
+		syslog(LOG_ALERT,
+		"%s is not authorized to change the password of %s",
+		user_context, name);
+		fprintf(stderr, _("%s: %s is not authorized to change the "
+			"password of %s\n"),
+		Prog, user_context, name);
+		freecon(user_context);
+		exit(1);
+	}
+
+#endif
+
 	/*
 	 * If the UID of the user does not match the current real UID,
 	 * check if I'm root.
 	 */
 	if (!amroot && pw->pw_uid != getuid ()) {
-#endif
 		fprintf (stderr,
 			 _
 			 ("%s: You may not view or modify password information for %s.\n"),
@@ -928,7 +987,7 @@ int main (int argc, char **argv)
 	}
 #endif				/* USE_PAM */
 	if (setuid (0)) {
-		fprintf (stderr, _("Cannot change ID to root.\n"));
+		fputs (_("Cannot change ID to root.\n"), stderr);
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		closelog ();
 		exit (E_NOPERM);
@@ -945,9 +1004,9 @@ int main (int argc, char **argv)
 	closelog ();
 	if (!qflg) {
 		if (!eflg)
-			printf (_("Password changed.\n"));
+			puts (_("Password changed."));
 		else
-			printf (_("Password set to expire.\n"));
+			puts (_("Password set to expire."));
 	}
 	exit (E_SUCCESS);
 	/* NOT REACHED */
