@@ -47,7 +47,7 @@
 
 #include <config.h>
 
-#ident "$Id: su.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: su.c 1743 2008-01-24 20:54:42Z nekral-guest $"
 
 #include <getopt.h>
 #include <grp.h>
@@ -164,7 +164,7 @@ static void su_failure (const char *tty)
 
 #ifdef USE_PAM
 /* Signal handler for parent process later */
-static void catch_signals (int sig)
+static void catch_signals (unused int sig)
 {
 	++caught;
 }
@@ -274,17 +274,17 @@ static void run_shell (const char *shellstr, char *args[], int doshell,
   */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: su [options] [LOGIN]\n"
-			   "\n"
-			   "Options:\n"
-			   "  -c, --command COMMAND         pass COMMAND to the invoked shell\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -, -l, --login                make the shell a login shell\n"
-			   "  -m, -p,\n"
-			   "  --preserve-environment        do not reset environment variables, and\n"
-			   "                                keep the same shell\n"
-			   "  -s, --shell SHELL             use SHELL instead of the default in passwd\n"
-			   "\n"));
+	fputs (_("Usage: su [options] [LOGIN]\n"
+	         "\n"
+	         "Options:\n"
+	         "  -c, --command COMMAND         pass COMMAND to the invoked shell\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -, -l, --login                make the shell a login shell\n"
+	         "  -m, -p,\n"
+	         "  --preserve-environment        do not reset environment variables, and\n"
+	         "                                keep the same shell\n"
+	         "  -s, --shell SHELL             use SHELL instead of the default in passwd\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
@@ -316,7 +316,7 @@ int main (int argc, char **argv)
 #else				/* !USE_PAM */
 	int err = 0;
 
-	RETSIGTYPE (*oldsig) ();
+	RETSIGTYPE (*oldsig) (int);
 	int is_console = 0;
 
 	struct spwd *spwd = 0;
@@ -362,19 +362,9 @@ int main (int argc, char **argv)
 		};
 
 		while ((c =
-			getopt_long (argc, argv, "-c:hlmps:", long_options,
+			getopt_long (argc, argv, "c:hlmps:", long_options,
 				     &option_index)) != -1) {
 			switch (c) {
-			case 1:
-				/* this is not an su option */
-				/* The next arguments are either '-', the
-				 * target name, or arguments to be passed
-				 * to the shell.
-				 */
-				/* rewind the (not yet handled) option */
-				optind--;
-				goto end_su_options;
-				break;	/* NOT REACHED */
 			case 'c':
 				command = optarg;
 				break;
@@ -399,7 +389,7 @@ int main (int argc, char **argv)
 				usage ();	/* NOT REACHED */
 			}
 		}
-	      end_su_options:
+
 		if (optind < argc && !strcmp (argv[optind], "-")) {
 			fakelogin = 1;
 			optind++;
@@ -878,7 +868,7 @@ int main (int argc, char **argv)
 #ifndef USE_PAM
 		(void) execve (shellstr, &argv[-1], environ);
 		err = errno;
-		(void) fprintf (stderr, _("No shell\n"));
+		(void) fputs (_("No shell\n"), stderr);
 		SYSLOG ((LOG_WARN, "Cannot execute %s", shellstr));
 		closelog ();
 		exit (err == ENOENT ? E_CMD_NOTFOUND : E_CMD_NOEXEC);
