@@ -29,12 +29,13 @@
 
 #include <config.h>
 
-#ident "$Id: sulogin.c,v 1.26 2006/05/12 22:32:14 kloczek Exp $"
+#ident "$Id: sulogin.c 1751 2008-02-03 16:28:03Z nekral-guest $"
 
 #include <fcntl.h>
 #include <pwd.h>
 #include <signal.h>
 #include <stdio.h>
+#include <sys/ioctl.h>
 #include "defines.h"
 #include "getdef.h"
 #include "prototypes.h"
@@ -60,7 +61,7 @@ extern char **environ;
 /* local function prototypes */
 static RETSIGTYPE catch_signals (int);
 
-static RETSIGTYPE catch_signals (int sig)
+static RETSIGTYPE catch_signals (unused int sig)
 {
 	exit (1);
 }
@@ -120,7 +121,7 @@ static RETSIGTYPE catch_signals (int sig)
 		}
 	}
 	if (access (PASSWD_FILE, F_OK) == -1) {	/* must be a password file! */
-		printf (_("No password file\n"));
+		puts (_("No password file"));
 #ifdef	USE_SYSLOG
 		SYSLOG (LOG_WARN, "No password file\n");
 		closelog ();
@@ -142,6 +143,12 @@ static RETSIGTYPE catch_signals (int sig)
 #endif
 		exit (1);	/* must be a terminal */
 	}
+	/* If we were init, we need to start a new session */
+	if (getppid() == 1) {
+		setsid();
+		if (ioctl(0, TIOCSCTTY, 1))
+			fputs (_("TIOCSCTTY failed"), stderr);
+	}
 	while (*envp)		/* add inherited environment, */
 		addenv (*envp++, NULL);	/* some variables change later */
 
@@ -164,7 +171,7 @@ static RETSIGTYPE catch_signals (int sig)
 			/*
 			 * Fail secure
 			 */
-			printf (_("No password entry for 'root'\n"));
+			puts (_("No password entry for 'root'"));
 #ifdef	USE_SYSLOG
 			SYSLOG (LOG_WARN, "No password entry for 'root'\n");
 			closelog ();
@@ -193,7 +200,7 @@ static RETSIGTYPE catch_signals (int sig)
 			SYSLOG (LOG_INFO, "Normal startup\n");
 			closelog ();
 #endif
-			puts ("\n");
+			puts ("");
 #ifdef	TELINIT
 			execl (PATH_TELINIT, "telinit", RUNLEVEL, (char *) 0);
 #endif
@@ -216,7 +223,7 @@ static RETSIGTYPE catch_signals (int sig)
 	signal (SIGALRM, SIG_DFL);
 	environ = newenvp;	/* make new environment active */
 
-	puts (_("Entering System Maintenance Mode\n"));
+	puts (_("Entering System Maintenance Mode"));
 #ifdef	USE_SYSLOG
 	SYSLOG (LOG_INFO, "System Maintenance Mode\n");
 #endif
