@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: userdel.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: userdel.c 1743 2008-01-24 20:54:42Z nekral-guest $"
 
 #include <errno.h>
 #include <fcntl.h>
@@ -99,14 +99,14 @@ static void remove_mailbox (void);
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: userdel [options] LOGIN\n"
-			   "\n"
-			   "Options:\n"
-			   "  -f, --force                   force removal of files,\n"
-			   "                                even if not owned by user\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -r, --remove                  remove home directory and mail spool\n"
-			   "\n"));
+	fputs (_("Usage: userdel [options] LOGIN\n"
+	         "\n"
+	         "Options:\n"
+	         "  -f, --force                   force removal of files,\n"
+	         "                                even if not owned by user\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -r, --remove                  remove home directory and mail spool\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
