@@ -29,7 +29,7 @@
 
 #include <config.h>
 
-#ident "$Id: usermod.c 1460 2007-11-24 22:41:24Z nekral-guest $"
+#ident "$Id: usermod.c 1772 2008-02-10 21:35:17Z nekral-guest $"
 
 #include <ctype.h>
 #include <errno.h>
@@ -87,21 +87,18 @@ static uid_t user_newid;
 static gid_t user_gid;
 static gid_t user_newgid;
 static char *user_comment;
+static char *user_newcomment;
 static char *user_home;
 static char *user_newhome;
 static char *user_shell;
+static char *user_newshell;
 static long user_expire;
+static long user_newexpire;
 static long user_inactive;
+static long user_newinactive;
 static long sys_ngroups;
 static char **user_groups;	/* NULL-terminated list */
 
-#ifdef WITH_AUDIT
-static char *user_newcomment;	/* Audit */
-static char *user_newshell;	/* Audit */
-static long user_newexpire;	/* Audit */
-static long user_newinactive;	/* Audit */
-#endif
-
 static char *Prog;
 
 static int
@@ -136,6 +133,8 @@ static int sgr_locked = 0;
 
 
 /* local function prototypes */
+static void date_to_str (char *buf, size_t maxsize,
+                         long int date, const char *negativ);
 static int get_groups (char *);
 static void usage (void);
 static void new_pwent (struct passwd *);
@@ -162,6 +161,25 @@ static void update_files (void);
 static void move_mailbox (void);
 #endif
 
+static void date_to_str (char *buf, size_t maxsize,
+                         long int date, const char *negativ)
+{
+	struct tm *tp;
+
+	if ((negativ != NULL) && (date < 0)) {
+		strncpy (buf, negativ, maxsize);
+	} else {
+		time_t t = date;
+		tp = gmtime (&t);
+#ifdef HAVE_STRFTIME
+		strftime (buf, maxsize, "%Y-%m-%d", tp);
+#else
+		snprintf (buf, maxsize, "%04d-%02d-%02d",
+		          tp->tm_year + 1900, tp->tm_mon + 1, tp->tm_mday);
+#endif				/* HAVE_STRFTIME */
+	}
+	buf[maxsize - 1] = '\0';
+}
 /*
  * Had to move this over from useradd.c since we have groups named
  * "56k-family"... ergh.
@@ -279,30 +297,30 @@ static int get_groups (char *list)
  */
 static void usage (void)
 {
-	fprintf (stderr, _("Usage: usermod [options] LOGIN\n"
-			   "\n"
-			   "Options:\n"
-			   "  -c, --comment COMMENT         new value of the GECOS field\n"
-			   "  -d, --home HOME_DIR           new home directory for the user account\n"
-			   "  -e, --expiredate EXPIRE_DATE  set account expiration date to EXPIRE_DATE\n"
-			   "  -f, --inactive INACTIVE       set password inactive after expiration\n"
-			   "                                to INACTIVE\n"
-			   "  -g, --gid GROUP               force use GROUP as new primary group\n"
-			   "  -G, --groups GROUPS           new list of supplementary GROUPS\n"
-			   "  -a, --append                  append the user to the supplemental GROUPS\n"
-			   "                                mentioned by the -G option without removing\n"
-			   "                                him/her from other groups\n"
-			   "  -h, --help                    display this help message and exit\n"
-			   "  -l, --login NEW_LOGIN         new value of the login name\n"
-			   "  -L, --lock                    lock the user account\n"
-			   "  -m, --move-home               move contents of the home directory to the\n"
-			   "                                new location (use only with -d)\n"
-			   "  -o, --non-unique              allow using duplicate (non-unique) UID\n"
-			   "  -p, --password PASSWORD       use encrypted password for the new password\n"
-			   "  -s, --shell SHELL             new login shell for the user account\n"
-			   "  -u, --uid UID                 new UID for the user account\n"
-			   "  -U, --unlock                  unlock the user account\n"
-			   "\n"));
+	fputs (_("Usage: usermod [options] LOGIN\n"
+	         "\n"
+	         "Options:\n"
+	         "  -c, --comment COMMENT         new value of the GECOS field\n"
+	         "  -d, --home HOME_DIR           new home directory for the user account\n"
+	         "  -e, --expiredate EXPIRE_DATE  set account expiration date to EXPIRE_DATE\n"
+	         "  -f, --inactive INACTIVE       set password inactive after expiration\n"
+	         "                                to INACTIVE\n"
+	         "  -g, --gid GROUP               force use GROUP as new primary group\n"
+	         "  -G, --groups GROUPS           new list of supplementary GROUPS\n"
+	         "  -a, --append                  append the user to the supplemental GROUPS\n"
+	         "                                mentioned by the -G option without removing\n"
+	         "                                him/her from other groups\n"
+	         "  -h, --help                    display this help message and exit\n"
+	         "  -l, --login NEW_LOGIN         new value of the login name\n"
+	         "  -L, --lock                    lock the user account\n"
+	         "  -m, --move-home               move contents of the home directory to the\n"
+	         "                                new location (use only with -d)\n"
+	         "  -o, --non-unique              allow using duplicate (non-unique) UID\n"
+	         "  -p, --password PASSWORD       use encrypted password for the new password\n"
+	         "  -s, --shell SHELL             new login shell for the user account\n"
+	         "  -u, --uid UID                 new UID for the user account\n"
+	         "  -U, --unlock                  unlock the user account\n"
+	         "\n"), stderr);
 	exit (E_USAGE);
 }
 
@@ -310,7 +328,7 @@ static void usage (void)
  * update encrypted password string (for both shadow and non-shadow
  * passwords)
  */
-static char *new_pw_passwd (char *pw_pass, const char *pw_name)
+static char *new_pw_passwd (char *pw_pass)
 {
 	if (Lflg && pw_pass[0] != '!') {
 		char *buf = xmalloc (strlen (pw_pass) + 2);
@@ -319,7 +337,7 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "updating passwd",
 			      user_newname, user_newid, 0);
 #endif
-		SYSLOG ((LOG_INFO, "lock user `%s' password", pw_name));
+		SYSLOG ((LOG_INFO, "lock user `%s' password", user_newname));
 		strcpy (buf, "!");
 		strcat (buf, pw_pass);
 		pw_pass = buf;
@@ -338,7 +356,7 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "updating password",
 			      user_newname, user_newid, 0);
 #endif
-		SYSLOG ((LOG_INFO, "unlock user `%s' password", pw_name));
+		SYSLOG ((LOG_INFO, "unlock user `%s' password", user_newname));
 		s = pw_pass;
 		while (*s) {
 			*s = *(s + 1);
@@ -349,7 +367,7 @@ static char *new_pw_passwd (char *pw_pass, const char *pw_name)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing password",
 			      user_newname, user_newid, 1);
 #endif
-		SYSLOG ((LOG_INFO, "change user `%s' password", pw_name));
+		SYSLOG ((LOG_INFO, "change user `%s' password", user_newname));
 		pw_pass = xstrdup (user_pass);
 	}
 	return pw_pass;
@@ -374,7 +392,7 @@ static void new_pwent (struct passwd *pwent)
 	}
 	if (!is_shadow_pwd)
 		pwent->pw_passwd =
-		    new_pw_passwd (pwent->pw_passwd, pwent->pw_name);
+		    new_pw_passwd (pwent->pw_passwd);
 
 	if (uflg) {
 #ifdef WITH_AUDIT
@@ -401,10 +419,8 @@ static void new_pwent (struct passwd *pwent)
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing comment",
 			      user_newname, user_newid, 1);
-		pwent->pw_gecos = user_newcomment;
-#else
-		pwent->pw_gecos = user_comment;
 #endif
+		pwent->pw_gecos = user_newcomment;
 	}
 
 	if (dflg) {
@@ -422,15 +438,10 @@ static void new_pwent (struct passwd *pwent)
 #ifdef WITH_AUDIT
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing user shell",
 			      user_newname, user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO, "change user `%s' shell from `%s' to `%s'",
 			 pwent->pw_name, pwent->pw_shell, user_newshell));
 		pwent->pw_shell = user_newshell;
-#else
-		SYSLOG ((LOG_INFO,
-			 "change user `%s' shell from `%s' to `%s'",
-			 pwent->pw_name, pwent->pw_shell, user_shell));
-		pwent->pw_shell = user_shell;
-#endif
 	}
 }
 
@@ -450,76 +461,30 @@ static void new_spent (struct spwd *spent)
 		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 			      "changing inactive days", user_newname,
 			      user_newid, 1);
+#endif
 		SYSLOG ((LOG_INFO,
 			 "change user `%s' inactive from `%ld' to `%ld'",
 			 spent->sp_namp, spent->sp_inact, user_newinactive));
 		spent->sp_inact = user_newinactive;
-#else
-
-		SYSLOG ((LOG_INFO,
-			 "change user `%s' inactive from `%ld' to `%ld'",
-			 spent->sp_namp, spent->sp_inact, user_inactive));
-		spent->sp_inact = user_inactive;
-#endif
 	}
 	if (eflg) {
-		/* XXX - dates might be better than numbers of days.  --marekm */
+		/* log dates rather than numbers of days. */
+		char new_exp[16], old_exp[16];
+		date_to_str (new_exp, sizeof(new_exp),
+		             user_newexpire * DAY, "never");
+		date_to_str (old_exp, sizeof(old_exp),
+		             user_expire * DAY, "never");
 #ifdef WITH_AUDIT
-		if (audit_fd >= 0) {
-			time_t exp_t;
-			struct tm *exp_tm;
-			char new_exp[16], old_exp[16];
-
-			if (user_newexpire == -1)
-				new_exp[0] = '\0';
-			else {
-				exp_t = user_newexpire * DAY;
-				exp_tm = gmtime (&exp_t);
-#ifdef HAVE_STRFTIME
-				strftime (new_exp, sizeof (new_exp), "%Y-%m-%d",
-					  exp_tm);
-#else
-				memset (new_exp, 0, sizeof (new_exp));
-				snprintf (new_exp, sizeof (new_exp) - 1,
-					  "%04i-%02i-%02i",
-					  exp_tm->tm_year + 1900,
-					  exp_tm->tm_mon + 1, exp_tm->tm_mday);
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+		              "changing expiration date", user_newname,
+		              user_newid, 1);
 #endif
-			}
-
-			if (user_expire == -1)
-				old_exp[0] = '\0';
-			else {
-				exp_t = user_expire * DAY;
-				exp_tm = gmtime (&exp_t);
-#ifdef HAVE_STRFTIME
-				strftime (old_exp, sizeof (old_exp), "%Y-%m-%d",
-					  exp_tm);
-#else
-				memset (old_exp, 0, sizeof (old_exp));
-				snprintf (old_exp, sizeof (old_exp) - 1,
-					  "%04i-%02i-%02i",
-					  exp_tm->tm_year + 1900,
-					  exp_tm->tm_mon + 1, exp_tm->tm_mday);
-#endif
-			}
-			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
-				      "changing expiration date", user_newname,
-				      user_newid, 1);
-		}
-
 		SYSLOG ((LOG_INFO,
-			 "change user `%s' expiration from `%ld' to `%ld'",
-			 spent->sp_namp, spent->sp_expire, user_newexpire));
+			 "change user `%s' expiration from `%s' to `%s'",
+			 spent->sp_namp, old_exp, new_exp));
 		spent->sp_expire = user_newexpire;
-#else
-		SYSLOG ((LOG_INFO,
-			 "change user `%s' expiration from `%ld' to `%ld'",
-			 spent->sp_namp, spent->sp_expire, user_expire));
-		spent->sp_expire = user_expire;
-#endif
 	}
-	spent->sp_pwdp = new_pw_passwd (spent->sp_pwdp, spent->sp_namp);
+	spent->sp_pwdp = new_pw_passwd (spent->sp_pwdp);
 	if (pflg)
 		spent->sp_lstchg = time ((time_t *) 0) / SCALE;
 }
@@ -568,7 +533,8 @@ static void update_group (void)
 		 * concurrent groups.
 		 */
 		was_member = is_on_list (grp->gr_mem, user_name);
-		is_member = Gflg && is_on_list (user_groups, grp->gr_name);
+		is_member = Gflg && (   (was_member && aflg)
+		                     || is_on_list (user_groups, grp->gr_name));
 
 		if (!was_member && !is_member)
 			continue;
@@ -609,17 +575,14 @@ static void update_group (void)
 			SYSLOG ((LOG_INFO, "delete `%s' from group `%s'",
 				 user_name, ngrp->gr_name));
 		} else if (!was_member && Gflg && is_member) {
-			ngrp->gr_mem = add_list (ngrp->gr_mem,
-						 lflg ? user_newname :
-						 user_name);
+			ngrp->gr_mem = add_list (ngrp->gr_mem, user_newname);
 			changed = 1;
 #ifdef WITH_AUDIT
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
 				      "adding user to group", user_name, -1, 1);
 #endif
 			SYSLOG ((LOG_INFO, "add `%s' to group `%s'",
-				 lflg ? user_newname : user_name,
-				 ngrp->gr_name));
+			         user_newname, ngrp->gr_name));
 		}
 		if (!changed)
 			continue;
@@ -667,6 +630,8 @@ static void update_gshadow (void)
 		 * concurrent groups.
 		 */
 		is_member = Gflg && is_on_list (user_groups, sgrp->sg_name);
+		is_member = Gflg && (   (was_member && aflg)
+		                     || is_on_list (user_groups, sgrp->sg_name));
 
 		if (!was_member && !was_admin && !is_member)
 			continue;
@@ -721,9 +686,7 @@ static void update_gshadow (void)
 				 "delete `%s' from shadow group `%s'",
 				 user_name, nsgrp->sg_name));
 		} else if (!was_member && Gflg && is_member) {
-			nsgrp->sg_mem = add_list (nsgrp->sg_mem,
-						  lflg ? user_newname :
-						  user_name);
+			nsgrp->sg_mem = add_list (nsgrp->sg_mem, user_newname);
 			changed = 1;
 #ifdef WITH_AUDIT
 			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
@@ -731,8 +694,7 @@ static void update_gshadow (void)
 				      user_newname, -1, 1);
 #endif
 			SYSLOG ((LOG_INFO, "add `%s' to shadow group `%s'",
-				 lflg ? user_newname : user_name,
-				 nsgrp->sg_name));
+			         user_newname, nsgrp->sg_name));
 		}
 		if (!changed)
 			continue;
@@ -827,14 +789,12 @@ static void process_flags (int argc, char **argv)
 		user_home = xstrdup (pwd->pw_dir);
 		user_shell = xstrdup (pwd->pw_shell);
 	}
-#ifdef WITH_AUDIT
 	user_newname = user_name;
 	user_newid = user_id;
 	user_newgid = user_gid;
 	user_newcomment = user_comment;
 	user_newhome = user_home;
 	user_newshell = user_shell;
-#endif
 
 #ifdef	USE_NIS
 	/*
@@ -862,10 +822,8 @@ static void process_flags (int argc, char **argv)
 		if (is_shadow_pwd && (spwd = getspnam (user_name))) {
 			user_expire = spwd->sp_expire;
 			user_inactive = spwd->sp_inact;
-#ifdef WITH_AUDIT
 			user_newexpire = user_expire;
 			user_newinactive = user_inactive;
-#endif
 		}
 	}
 
@@ -907,11 +865,7 @@ static void process_flags (int argc, char **argv)
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
-#ifdef WITH_AUDIT
 				user_newcomment = optarg;
-#else
-				user_comment = optarg;
-#endif
 				cflg++;
 				break;
 			case 'd':
@@ -926,38 +880,21 @@ static void process_flags (int argc, char **argv)
 				break;
 			case 'e':
 				if (*optarg) {
-#ifdef WITH_AUDIT
 					user_newexpire = strtoday (optarg);
 					if (user_newexpire == -1) {
-#else /* } */
-					user_expire = strtoday (optarg);
-					if (user_expire == -1) {
-#endif
 						fprintf (stderr,
 							 _
 							 ("%s: invalid date '%s'\n"),
 							 Prog, optarg);
 						exit (E_BAD_ARG);
 					}
-#ifdef WITH_AUDIT
 					user_newexpire *= DAY / SCALE;
-#else
-					user_expire *= DAY / SCALE;
-#endif
 				} else
-#ifdef WITH_AUDIT
 					user_newexpire = -1;
-#else
-					user_expire = -1;
-#endif
 				eflg++;
 				break;
 			case 'f':
-#ifdef WITH_AUDIT
 				user_newinactive = get_number (optarg);
-#else
-				user_inactive = get_number (optarg);
-#endif
 				fflg++;
 				break;
 			case 'g':
@@ -983,15 +920,7 @@ static void process_flags (int argc, char **argv)
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
-
-				/*
-				 * If the name does not really change, we mustn't
-				 * set the flag as this will cause rather serious
-				 * problems later!
-				 */
-				if (strcmp (user_name, optarg))
-					lflg++;
-
+				lflg++;
 				user_newname = optarg;
 				break;
 			case 'L':
@@ -1014,11 +943,7 @@ static void process_flags (int argc, char **argv)
 						 Prog, optarg);
 					exit (E_BAD_ARG);
 				}
-#ifdef WITH_AUDIT
 				user_newshell = optarg;
-#else
-				user_shell = optarg;
-#endif
 				sflg++;
 				break;
 			case 'u':
@@ -1039,6 +964,40 @@ static void process_flags (int argc, char **argv)
 		fprintf (stderr, _("%s: no flags given\n"), Prog);
 		exit (E_USAGE);
 	}
+
+	if (user_newid == user_id) {
+		uflg = 0;
+		oflg = 0;
+	}
+	if (user_newgid == user_gid) {
+		gflg = 0;
+	}
+	if (strcmp (user_newshell, user_shell) == 0) {
+		sflg = 0;
+	}
+	if (strcmp (user_newname, user_name) == 0) {
+		lflg = 0;
+	}
+	if (user_newinactive == user_inactive) {
+		fflg = 0;
+	}
+	if (user_newexpire == user_expire) {
+		eflg = 0;
+	}
+	if (strcmp (user_newhome, user_home) == 0) {
+		dflg = 0;
+		mflg = 0;
+	}
+	if (strcmp (user_newcomment, user_comment) == 0) {
+		cflg = 0;
+	}
+
+	if (Uflg + uflg + sflg + pflg + oflg + mflg + Lflg + lflg + Gflg +
+	    gflg + fflg + eflg + dflg + cflg == 0) {
+		fprintf (stderr, _("%s: no changes\n"), Prog);
+		exit (E_SUCCESS);
+	}
+
 	if (!is_shadow_pwd && (eflg || fflg)) {
 		fprintf (stderr,
 			 _
@@ -1082,12 +1041,6 @@ static void process_flags (int argc, char **argv)
 		exit (E_USAGE);
 	}
 
-	if (dflg && strcmp (user_home, user_newhome) == 0)
-		dflg = mflg = 0;
-
-	if (uflg && user_id == user_newid)
-		uflg = oflg = 0;
-
 	/* local, no need for xgetpwnam */
 	if (lflg && getpwnam (user_newname)) {
 		fprintf (stderr, _("%s: user %s exists\n"), Prog, user_newname);
@@ -1146,7 +1099,9 @@ static void close_files (void)
 	pw_locked = 0;
 	spw_locked = 0;
 	gr_locked = 0;
+#ifdef	SHADOWGRP
 	sgr_locked = 0;
+#endif
 
 	/*
 	 * Close the DBM and/or flat files
@@ -1328,8 +1283,8 @@ static void move_home (void)
 					fail_exit (E_HOMEDIR);
 				}
 				if (copy_tree (user_home, user_newhome,
-					       uflg ? user_newid : -1,
-					       gflg ? user_newgid : -1) == 0) {
+					       uflg ? (long int)user_newid : -1,
+					       gflg ? (long int)user_newgid : -1) == 0) {
 					if (remove_tree (user_home) != 0 ||
 					    rmdir (user_home) != 0)
 						fprintf (stderr,
